/* ====================================================================
 * The Vovida Software License, Version 1.0
 *
 * Copyright (c) 1999, 2000 Vovida Networks, Inc.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by Vovida Networks,
 *        Inc. (http://www.vovida.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must 
 *    not be used to endorse or promote products derived from this 
 *    software without prior written permission. For written permission, 
 *    please contact vocal@vovida.org.
 *
 * 5. Products derived from this software may not be called "VOCAL",
 *    nor may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL VOVIDA NETWORKS, INC. OR ITS 
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by Vovida 
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see 
 * <http://www.vovida.org/>.
 *
 */

/* Authors: Eddie Mendonca
*/

/* Includes */
#include <stdio.h>
#include <string.h>
#include <strstream.h>

#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>

#include <getopt.h>
#include <iostream>
#include <cassert>
#include <FileDataStore.hxx>
#include <VIoException.hxx>

#include <lber.h>
#include <ldap.h>

#include "ldap2vocaluser.hxx"

/* Initialize LDAP session */
void init(LDAP **ld, char *ldaphost, int ldapport)
{
    if ((*ld = ldap_init(ldaphost, ldapport) ) == NULL )
    {
        perror( "ldap_init" );
    }
}

/*  Authenticate to LDAP server as nobody */
void auth(LDAP *ld, char *binddn, char *bindpw)
{
    if (ldap_simple_bind_s(ld, binddn, bindpw) != LDAP_SUCCESS)
    {
        ldap_perror( ld, "ldap_simple_bind_s" );
        return ;
    }
}

/*  Report string on LDAP error */
void report_ldap_error( char *s, int err )
{
    char *msg;

    msg = ldap_err2string( err );
    fprintf(stderr, "An LDAP error occurred while %s\n(%s)\n", s, msg );
}

/* Set new search size limit to infinite (0) */
void set_sizelimit(LDAP *ld)
{
    int sizelimit = 0;

#ifdef LDAP_OPT_ON      /* Netscape SDK */
    if (ldap_set_option(ld, LDAP_OPT_SIZELIMIT, &sizelimit) != 0)
    {
        ldap_perror( ld, "ldap_set_option" );
    }
#else                   /* UMICH SDK */
    ld->ld_sizelimit &= ~sizelimit;
#endif
}

/*  Create XML file for given entry */
char *create_xmlbuffer(char *phonenumber, char *ip, char *fna, char *cfb)
{
    char *buffer = new char[MAX_FILE_SIZE];
    strstream buf(buffer, MAX_FILE_SIZE);

    printf("\nExporting: %s\n", phonenumber);

    buf << "<user><name>" << phonenumber << "</name>" << endl
    << "<password>6f7d5634f65192a3b9c8479cee3b655</password>" << endl
    << "<ipaddr>" << ip << "</ipaddr>" << endl
    << "<marshalgroup>UserAgentGroup</marshalgroup>" << endl
    << "<staticRegistration value=\"false\"><terminatingContactList><contact value=\"UserAgentGroup\"></contact>" << endl
    << "<contact value=\"xxx.xxx.xxx.xxx:yyyy\"></contact>" << endl
    << "</terminatingContactList>" << endl
    << "</staticRegistration>" << endl
    << "<authenticationType value=\"Access list\"><authenticationPassword value=\"password\"></authenticationPassword>" << endl
    << "</authenticationType>" << endl
    << "<failurecase>unknown</failurecase>" << endl
    << "<cfa enabled=\"false\"><setfeat>OFF</setfeat>" << endl
    << "<featuregroup>ForwardAllCallsGroup</featuregroup>" << endl
    << "<forwardto>none</forwardto>" << endl
    << "</cfa>" << endl
    << "<fnab enabled=\"false\"><setfeat>OFF</setfeat>" << endl
    << "<featuregroup>ForwardNoAnswerBusyGroup</featuregroup>" << endl
    << "<fna set=\"false\"><forwardto>" << fna << "</forwardto>" << endl
    << "</fna>" << endl
    << "<cfb set=\"false\"><forwardto>" << cfb << "</forwardto>" << endl
    << "</cfb>" << endl
    << "</fnab>" << endl
    << "<cs enabled=\"false\"><setfeat>OFF</setfeat>" << endl
    << "<featuregroup>CallScreeningGroup</featuregroup>" << endl
    << "</cs>" << endl
    << "<clbl enabled=\"false\"><setfeat>OFF</setfeat>" << endl
    << "<featuregroup>CallBlockingGroup</featuregroup>" << endl
    << "<nineHundredBlocked adminSet=\"false\" userSet=\"false\"></nineHundredBlocked>" << endl
    << "<longDistanceBlocked adminSet=\"false\" userSet=\"false\"></longDistanceBlocked>" << endl
    << "</clbl>" << endl
    << "<jtapi enabled=\"true\"><setfeat>ON</setfeat>" << endl
    << "<featuregroup>JtapiGroup</featuregroup>" << endl
    << "</jtapi>" << endl
    << "<callReturn enabled=\"false\"><setfeat>OFF</setfeat>" << endl
    << "<featuregroup>CallReturnGroup</featuregroup>" << endl
    << "</callReturn>" << endl
    << "<callerIdBlocking enabled=\"false\"><setfeat>OFF</setfeat>" << endl
    << "<featuregroup>CallerIdBlockingGroup</featuregroup>" << endl
    << "</callerIdBlocking>" << endl
    << "<voicemail enabled=\"false\"><setfeat>OFF</setfeat>" << endl
    << "<featuregroup>VoiceMailGroup</featuregroup>" << endl
    << "</voicemail>" << endl
    << "<aliases></aliases>" << endl
    << "</user>" << endl

    << ends;

    return (buffer);
}


/* Strip spaces from telephone number */
char *strip_spaces(char *oldphonenumber)
{
    char *newphonenumber = new char[MAX_ATTR_STRING];
    char *ptr = oldphonenumber;
    char *ptrnew = newphonenumber;

    while (*ptr != '\0')
    {
        if (*ptr != ' ')
            *ptrnew++ = *ptr++;
        else
            ptr++;
    }
    *ptrnew++ = '\0';
    return (newphonenumber);
}

/*  Find matches */
int export_ldap(LDAP *ld, char *basedn, char *outputdir)
{
    LDAPMessage *result, *e;
    int rc, msgid, finished;
    int num_entries = 0;
    char *filebuffer;
    char **attr_telephonenumber;
    char **attr_vocalPhonenumber;
    char **attr_vocalIP;
    char **attr_vocalFNA;
    char **attr_vocalCFB;
    char *rawphonenumber;
    char *phonenumber;
    char *ip;
    char *fna;
    char *cfb;

    FileDataStore f(HASH_KEY, outputdir);

    /* search */
    if ((msgid = ldap_search(ld, basedn, LDAP_SCOPE_SUBTREE,
                             SEARCH_FILTER, NULL, 0 )) < 0 )
    {
        ldap_perror( ld, "ldap_search" );
        return ( 1 );
    }

    /* Loop, polling for results until finished */
    finished = 0;
    while ( !finished )
    {
        result = NULL;
        rc = ldap_result( ld, msgid, 0, NULL, &result );
        switch ( rc )
        {
            case - 1:
            /* some error occurred */
            ldap_perror( ld, "ldap_result" );
            return ( 1 );
            case 0:
            /* Timeout was exceeded.  No entries are ready for retrieval. */
            if ( result != NULL )
            {
                ldap_msgfree( result );
            }
            break;
            default:
            /*
             * Either an entry is ready for retrieval, or all entries have
             * been retrieved.
             */
            if (( e = ldap_first_entry( ld, result )) == NULL )
            {
                /* All done */
                finished = 1;
                if ( result != NULL )
                {
                    ldap_msgfree( result );
                }
                continue;
            }

            /* for each entry ... */
            if ((attr_telephonenumber =
                        ldap_get_values(ld, e, ATTR_TELEPHONENUMBER)) != NULL)
            {
                num_entries++;
                rawphonenumber = strdup(attr_telephonenumber[0]);

                if ((attr_vocalPhonenumber =
                            ldap_get_values(ld, e, ATTR_VOCAL_PHONENUMBER)) != NULL)
                    rawphonenumber = strdup(attr_vocalPhonenumber[0]);
                else
                    rawphonenumber = PHONE_NOT_IN_LDAP;

                if ((attr_vocalIP = ldap_get_values(ld, e, ATTR_VOCAL_IP)) != NULL)
                    ip = strdup(attr_vocalIP[0]);
                else
                    ip = IP_NOT_IN_LDAP;

                if ((attr_vocalFNA = ldap_get_values(ld, e, ATTR_VOCAL_FNA)) != NULL)
                    fna = strdup(attr_vocalFNA[0]);
                else
                    fna = FNA_NOT_IN_LDAP;

                if ((attr_vocalCFB = ldap_get_values(ld, e, ATTR_VOCAL_CFB)) != NULL)
                    cfb = strdup(attr_vocalCFB[0]);
                else
                    cfb = CFB_NOT_IN_LDAP;

                /* Strip any spaces from LDAP phone number */
                phonenumber = strip_spaces(rawphonenumber);

                /* Set filebuffer to xml output */
                filebuffer = create_xmlbuffer(phonenumber, ip, fna, cfb);

                /* Create a vocal user file for each entry */
                f.putItem(ACCOUNTS_GROUP, phonenumber, filebuffer);

                delete filebuffer;
                delete phonenumber;
                ldap_value_free(attr_vocalPhonenumber);
                ldap_value_free(attr_vocalIP);
                ldap_value_free(attr_vocalFNA);
                ldap_value_free(attr_vocalCFB);
            }
            ldap_msgfree( result );
        }
        /* Call anything here while searching */
    }
    printf ("\nExported %d entries to %s\n", num_entries, outputdir);
    return (0);
}

/* Check to see if user can create output directory */
int check_outputdir(char *outputdir)
{
    int i = 0;
    /* Add trailing slash to outputdir */
    while (outputdir[i++] != '\0');
    if (outputdir[i - 2] != '/' && outputdir[i - 2] != '\\')
    {
        outputdir[i - 1] = '/';
        outputdir[i] = '\0';
    }

    if (!opendir(outputdir))
    {
        if (!mkdir(outputdir, DIR_PERMISSIONS))
            return 1;
        else
        {
            printf("Error creating output directory: %s \n", outputdir);
            printf("Check permissions \n");
            return 0;
        }
    }
    return 1;
}

/* MAIN */
main(int argc, char **argv)
{
    int i;
    int passedhost = 0, passedbasedn = 0, passedoutputdir = 0;
    char *usage = "Usage: %s -h ldaphost -b basedn -d output_directory [-p ldap_port] [-D binddn] [-w password]\n";
    int ldapport = 0;
    char *prog = NULL;
    char *ldaphost = NULL;
    char *basedn = NULL;
    char *outputdir = NULL;
    char *binddn = NULL;
    char *bindpw = NULL;
    LDAP *ld;

    if (( prog = strrchr( argv[0], '/' )) == NULL )
    {
        prog = argv[0];
    }
    else
    {
        ++prog;
    }

    while ((i = getopt( argc, argv, "h:p:b:d:D:w:" )) != EOF)
    {
        switch ( i )
        {
            case 'h':   /* LDAP host */
            ldaphost = strdup( optarg );
            passedhost = 1;
            break;
            case 'p':   /* LDAP port */
            ldapport = atoi( optarg );
            break;
            case 'b':   /* LDAP search root */
            basedn = strdup( optarg );
            passedbasedn = 1;
            break;
            case 'd':   /* Output directory */
            outputdir = strdup( optarg );
            passedoutputdir = check_outputdir(outputdir);
            break;
            case 'D':   /* Bind as Distingished Name */
            binddn = strdup( optarg );
            break;
            case 'w':   /* Bind password */
            bindpw = strdup( optarg );
            break;
            default:
            fprintf( stderr, usage, prog );
            exit( 1 );
        }
    }
    /* Set ldapport to default of 389, if not supplied */
    if (!ldapport) ldapport = LDAP_PORT;

    /* Check for all the required arguments */
    if (!(passedhost && passedbasedn && passedoutputdir))
    {
        fprintf( stderr, usage, prog );
        exit( 1 );
    }

    init(&ld, ldaphost, ldapport);
    auth(ld, binddn, bindpw);
    set_sizelimit(ld);
    printf("Exporting Users, Please wait...\n");
    if ( (i = export_ldap(ld, basedn, outputdir)) )
        printf("Error: export_ldap returned: %d\n", i);
    ldap_unbind(ld);
}
