#!/usr/bin/perl -w
#
# script to randomly select a calling user and a called user,
#        fix up their respectively ua.cfg files, launch respective
#        ua's, let ua make x calls, then select another pair of
#        user accounts and repeat the whole process....
#
# Usage: ranGenLoad rand-offset rand-limit calls-per-run
#
# Copyright 2000 by Wendy Breu 
#
#
# $Id: ranLoadGen.pl,v 1.1 2000/08/04 20:30:32 wbreu Exp $
#
#
###############################################################################
# display usage message
#
if (@ARGV != 4 ) {
  die "Usage:  $0  rand-base rand-limit calls-per-run times-to-run\n";
}

# verbose mode
$verbose = 1;

# parse arguments
$randbase = $ARGV[0];
$randlimit = $ARGV[1];
$callsprun = $ARGV[2];
$times2run = $ARGV[3];

# set default calling/ called user ids:
$calling = 10000;
$called  = 10001;

# set default paths where calling/ called ua to be invoked:
$callingpath = "/root/ldgen/call/ua.call.cfg";
$calledpath  = "/root/ldgen/rec/ua.rec.cfg";

# setup path to ua:
$uapath = "/root/ldgen/ua"; 

# seed the random number generator
srand( time() ^ ($$ + ($$ << 15)) );

# generate calling/ called users a random
sub genUsers {
   $range = $randlimit - $randbase;  # 0..n
   $calling = int(rand $range) + $randbase;
   $called  = int(rand $range) + $randbase;
   printf( "calling = %d, called = %d\n", $calling, $called ) if $verbose;
   
   if ($calling eq $called) {
      $called = $called - 1;
      printf( "calling = %d, called = %d\n", $calling, $called ) if $verbose;
   }
}

# fix up the ua.cfg files, called by prepareData()
sub fixUaCfg {
   $clg = $_[0];    # calling number
   $cld = $_[1];    # called number

   # assume <UA> and <OUTFILE> are already opened:
   while (<UA>)
   {
       $line = $_;
       chomp($line);
       if (/(^User_Name)/) {
          @fields = split(/\s+/, $line);
          printf( "f1=%s f2=%s f3=%s\n", $fields[0], $fields[1], $fields[2] ) if $verbose;
          $newline = "$fields[0]     $fields[1]     $clg";
          print OUTFILE "$newline\n"; 
          printf( "newline = $newline\n" );
          next;
       }
       if (/(^CallUrl)/) {
          @fields = split(/\s+/, $line);
          printf( "f1=%s f2=%s f3=%s\n", $fields[0], $fields[1], $fields[2] ) if $verbose;
          @sipurl = split(/@/, $fields[2]);
          $newline = "$fields[0]     $fields[1]     sip:$cld@".$sipurl[1];
          print OUTFILE "$newline\n"; 
          printf( "newline = $newline\n" );
          next;
       }
       if (/(^NumOfCalls)/) {
          @fields = split(/\s+/, $line);
          printf( "f1=%s f2=%s f3=%s\n", $fields[0], $fields[1], $fields[2] ) if $verbose;
          $newline = "$fields[0]     $fields[1]     $callsprun";
          print OUTFILE "$newline\n"; 
          printf( "newline = $newline\n\n" );
          next;
       }
       # ignore the rest of cfg data, just output to OUTFILE:
       print OUTFILE "$line\n";
   }
}

# set up data used to fix up the ua config files:
sub prepareData {
   $temp = "temp.out";
   if (-e $temp )
   {
      rmdir $temp;
   }

   # first, fix up the calling ua.cfg
   open( UA, "<" . $callingpath) || die "Could not open $callingpath\n";
   open(OUTFILE, ">$temp");
      fixUaCfg( $calling, $called );
   close( UA );
   close( OUTFILE );
   rmdir  "$callingpath".".bak";
   rename $callingpath, "$callingpath".".bak";
   rename $temp, $callingpath;

   # now, fix up the called ua.cfg
   rmdir $temp;
   open(UA, "<" . $calledpath ) || die "Could not open $calledpath\n";
   open(OUTFILE, ">$temp");
       fixUaCfg( $called, $calling );
   close(UA);
   close( OUTFILE );
   rmdir  "$calledpath".".bak";
   rename $calledpath, "$calledpath".".bak";
   rename $temp, $calledpath;
}

# launch ua -f ua.cfg respectively
sub startUas {
   system("./start-uaxterms startupUa.sxt");
   print "done calling start-xterms\n";
   sleep $callsprun*(rand(20)+1);
   print "done sleeping\n";
}

# main prog
sub main {
   # loop $times2run
   $i = $times2run;
   while ($i) {
       genUsers();
       prepareData();     
       startUas();
       $i--;           # comment this out if run forever
   }   
}

main();

## end of program
