/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const LSSendOnlySession_cxx_Version = 
"$Id: LSSendOnlySession.cxx,v 1.5 2000/11/09 02:19:27 surendra Exp $";

#include <strstream.h>

#include "LSSendOnlySession.hxx"
#include "TRIPOpenMsg.hxx"
#include "TRIPErrorHandler.hxx"

void 
LSSendOnlySession::dump(ostream& os) const
{
    os << "***LSSendOnly***" << endl;
    LSPeerSession::dump(os);
}

void
LSSendOnlySession::sendOpen() throw (VNetworkException&)
{
    assert(_connection.isLive());
    ///Send OPEN MSG
    cpLog(LOG_DEBUG, "Sending OPEN Msg.");
    TRIPOpenMsg msg(_parentItad, _tripId.getId(), 4*60);
    Sptr<TRIPCapabilityInfo>  cInfo = new TRIPCapabilityInfo();

    Sptr<TRIPTransCapability> cap = new TRIPTransCapability(TRIP_TRANS_SND); 
    cInfo->addCapability(cap);

    Sptr<TRIPRouteCapability> rCap = new TRIPRouteCapability();
    Sptr<TRIPRouteHdr> rHdr1 = new TRIPRouteHdr(TRIP_ADDR_POTS, TRIP_APP_PROTO_SIP);
    rCap->addRouteCapability(rHdr1);
    cInfo->addCapability(rCap);

    msg.addParameter(cInfo);
    char buf[4096];
    strstream oStr(buf, 4096);
    oStr << msg << endl << ends;
    cpLog(LOG_DEBUG, "%s", oStr.str());
    msg.writeData(_connection.getConnId());
}


void
LSSendOnlySession::handleUpdateMessage(Sptr<TRIPMsg> msg)
{
    //A send only Session can not process a update
    //Message ans so should send a notification message to
    //the peer and go to the idle state.
    string data("SendOnly session received update message");
    if(msg != 0)
    {
        sendNotification(TRIP_ERR_UPDATE, 6, data);
        _state->fail(*this);
    }
}

