#ifndef LSSessionManager_hxx
#define LSSessionManager_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const LSSessionManager_hxx_Version = 
"$Id: LSSessionManager.hxx,v 1.5 2000/11/09 02:19:36 surendra Exp $";

#include "global.h"
#include <vector>
#include <string>

#include "vtypes.h"
#include "Sptr.hxx"
#include "TRIPId.hxx"
#include "Connection.hxx"
#include "TRIPBadDataException.hxx"
#include "TRIPTransCapability.hxx"

class LSPeerSession;

/** 
    Singelton class.
    LSSessionManager implements the  connection management at the LS.It is listening
    on port 6069 for any incoming open request.When a request arrives, parses it find
    the capabilities.Based on the capabilities creates a LSPeerSession object and 
    delegates the new connection to the LSPeerSession.
*/


class LSSessionManager {
    public:
        ///
        typedef vector<Sptr<LSPeerSession> >  LSPeerList;

        ///
        static LSSessionManager& instance(u_int16_t itad=0, TRIPTransMode tMode = TRIP_TRANS_SND_RCV);

        ///
        string className() { return "LSSessionManager"; }

        ///Destructor
        virtual ~LSSessionManager();

        /**
           Frees the memory associated with singelton instance.
           gets register to atexit() function at the time of creation.
        */
        static void destroy(void);

        ///
        friend ostream& operator<<(ostream& os, const LSSessionManager& lslocator);

        ///Returns true if session with given ITAD and tripId exists
        bool locateExistingPeerSession(u_int16_t itad, const TRIPId& tripId);

        ///Returns true if a PeerSession exists for a given destination
        Sptr<LSPeerSession>  locateExistingPeerSession(const string& destIp);


        ///
        u_int16_t getItad() const { return _itad; };

        ///
        TRIPTransMode getTransMode() const { return _transMode; };

        ///Runs in while loop accepting OPEN connections in forground in the main thread.
        void run();

        ///Creates a separate thread to run the while loop accepting connections.
        void threadRun();

        ///
        const LSPeerList& getSessionList() const { return _sessionList; };


    protected:
        static void* threadRunImpl(void* args);

         ///
         void processMsg();

         ///
         void processOpenRequest(Connection& conn) throw (VNetworkException&, TRIPBadDataException&);

        ///
        void init();


         ///
         LSSessionManager(u_int16_t itad, TRIPTransMode tMode);


         ///
         static LSSessionManager* _instance;

         ///
         Sptr<TcpServerSocket>  _serverSocket;

         ///
         Sptr<VThread>          _thread;

         ///
         LSPeerList _sessionList;

         u_int16_t _itad;
         TRIPId    _tripId;

         TRIPTransMode _transMode;
};

#endif
