
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const TRIPState_cxx_Version = 
"$Id: TRIPState.cxx,v 1.8 2000/11/08 03:21:01 kevin Exp $";
#include <stdio.h>

#include "TRIPState.hxx"
#include "cpLog.h"
#include "TRIPStateUnknown.hxx"
#include "TRIPStateIdle.hxx"
#include "TRIPStateOpenSent.hxx"
#include "TRIPStateConnect.hxx"
#include "TRIPStateOpenConfirm.hxx"
#include "TRIPStateEstablished.hxx"
#include "TRIPStateActive.hxx"
#include "TRIPErrorHandler.hxx"
#include "LSPeerSession.hxx"

void 
TRIPState::error(const string& errMsg)
{
    char buf[256];
    sprintf(buf, "****Invalid state transition, %s", errMsg.c_str());
    cpLog(LOG_ALERT, buf);
    //throw TRIPInvalidStateException(buf, __FILE__, __LINE__, 0);
}

void
TRIPState::info(const string& errMsg)
{
    char buf[256];
    sprintf(buf, "****Information, %s", errMsg.c_str());
    cpLog(LOG_ALERT, buf);
}


void 
TRIPState::start(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    // start is ignored in all states other than idle.
    info(className()+"::start");
}

void
TRIPState::connect(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    // connect is ignored in all states other than idle.
    info(className()+"::connect");
}

void 
TRIPState::fail(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    // In case of matching connections between an LS pair
    session.sendNotification(TRIP_ERR_CEASE, 0, "");
  
    error(className()+"::fail");
    idle(session);
}

void 
TRIPState::connectRetryTimerExpired(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    session.sendNotification(TRIP_ERR_FSM, 0, "");
    error(className()+"::connectRetryTimerExpired");
    idle(session);
}

void
TRIPState::holdTimerExpired(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    session.sendNotification(TRIP_ERR_HOLD_TIMER, 0, "");
    error(className() + "::holdTimerExpired");
    idle(session);
}


void 
TRIPState::sendOpen(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    session.sendNotification(TRIP_ERR_FSM, 0, "");
    error(className()+"::sendOpen");
    idle(session);
}

void 
TRIPState::receiveOpen(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    session.sendNotification(TRIP_ERR_FSM, 0 , "");
    error(className()+"::receiveOpen");
    idle(session);
}

void 
TRIPState::receiveKeepAlive(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    session.sendNotification(TRIP_ERR_FSM, 0 , "");
    error(className()+"::receiveKeepAlive(");
    idle(session);
}

void 
TRIPState::receiveNotification(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    session.sendNotification(TRIP_ERR_FSM, 0 , "");
    error(className()+"::receiveNotification");
    idle(session);
}

void 
TRIPState::lostConnection(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    error(className()+"::lostConnection");
    idle(session);
}

void 
TRIPState::stop(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    session.sendNotification(TRIP_ERR_CEASE, 0, "");
    error(className()+"::stop");
    idle(session);
}

void 
TRIPState::changeState(LSPeerSession& session, TRIPState* newState) 
{
    cpLog(LOG_DEBUG, "TRIPState::CHANGEState");
    session.setState(newState);
}

void
TRIPState::idle(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    cpLog(LOG_DEBUG, "TRIPState::idle");
    session.closeConnections();
    session.releaseTRIPResources();
    session.setRestartTimer();
    changeState(session, new TRIPStateIdle);
}

