
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const TRIPStateConnect_cxx_Version = 
"$Id: TRIPStateConnect.cxx,v 1.7 2000/11/08 18:55:04 kevin Exp $";
#include <stdio.h>

#include "TRIPStateConnect.hxx"
#include "TRIPStateOpenSent.hxx"
#include "TRIPStateActive.hxx"
#include "cpLog.h"
#include "LSPeerSession.hxx"


void 
TRIPStateConnect::sendOpen(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    cpLog(LOG_DEBUG, "TRIPStateConnect::sendOpen");
    try
    {
	session.cancelConnectRetryTimer();
        session.sendOpen();
        session.setHoldTimer();
        changeState(session, new TRIPStateOpenSent);
    }
    catch(VException& e)
    {
        cpLog(LOG_DEBUG, "Failed to send open, reason %s",
                          e.getDescription().c_str());
	session.setConnectRetryTimer();
        changeState(session, new TRIPStateActive);
    }
}

void
TRIPStateConnect::connect(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    cpLog(LOG_DEBUG, "TRIPStateConnect::connect");
    session.setConnectRetryTimer();
    session.makeConnection();
}


void
TRIPStateConnect::lostConnection(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    cpLog(LOG_DEBUG, "TRIPStateConnect::lostConnection");
    session.closeConnections();
    session.setConnectRetryTimer();
    changeState(session, new TRIPStateActive);
}

void
TRIPStateConnect::connectRetryTimerExpired(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    try
    {
        cpLog(LOG_DEBUG, "TRIPStateConnect::connectRetryTimerExpired");
        session.closeConnections();
        session.setConnectRetryTimer();
        session.makeConnection();
    }
    catch(VNetworkException &e)
    {   
        cpLog(LOG_DEBUG, "Network error, reason:%s",
                          e.getDescription().c_str());
 	changeState(session, new TRIPStateActive);
	
    }

}

