
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const TRIPStateOpenSent_cxx_Version = 
"$Id: TRIPStateOpenSent.cxx,v 1.5 2000/11/09 02:30:28 kevin Exp $";
#include <stdio.h>

#include "TRIPState.hxx"
#include "TRIPStateOpenSent.hxx"
#include "TRIPStateOpenConfirm.hxx"
#include "TRIPStateActive.hxx"
#include "cpLog.h"
#include "LSPeerSession.hxx"
#include "TRIPErrorHandler.hxx"


void 
TRIPStateOpenSent::receiveOpen(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    cpLog(LOG_DEBUG, "TRIPStateOpenSent::receiveOpen");
    session.setHoldTimer();
    changeState(session, new TRIPStateOpenConfirm);
}

void
TRIPStateOpenSent::lostConnection(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    cpLog(LOG_DEBUG, "TRIPStateOpenSent::lostConnection");
    session.closeConnections();
    session.setConnectRetryTimer();
    changeState(session, new TRIPStateActive);
}

void
TRIPStateOpenSent::holdTimerExpired(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    cpLog(LOG_DEBUG, "TRIPStateOpenSent::holdTimerExpired");
    session.sendNotification(TRIP_ERR_HOLD_TIMER, 0, "");
    idle(session);
}

void
TRIPStateOpenSent::stop(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    cpLog(LOG_DEBUG, "TRIPStateOpenSent::stop");
    session.sendNotification(TRIP_ERR_CEASE, 0, "");
    idle(session);
}

void
TRIPStateOpenSent::fail(LSPeerSession& session) throw (TRIPInvalidStateException&)
{
    cpLog(LOG_DEBUG, "TRIPStateOpenSent::fail");
    session.sendNotification(TRIP_ERR_OPEN, 0, "");
    idle(session);
}


