/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "TRIPAttrHeader.hxx"
#include "TRIPUtil.hxx"
#include "cpLog.h"

TRIPAttrHeader::TRIPAttrHeader( u_int8_t aFlg, TRIPAttrTypeFlag type)
{
    _dataRep._flag = aFlg;
    _dataRep._type = type;
    _dataRep._length = 0;
}

TRIPAttrHeader::TRIPAttrHeader(const char* buf) throw(TRIPBadDataException&)
{
    memcpy(&_dataRep, buf, 4);
    _dataRep._length = ntohs(_dataRep._length);

    if(_dataRep._flag &  TRIP_FLG_MAX || (_dataRep._type <= 0 && _dataRep._type >= TRIP_ATTR_TYPE_MAX))
    {
        cpLog(LOG_ALERT, "BAD Attribute header data");
        throw TRIPBadDataException("BAD Attribute header data", __FILE__, __LINE__, 0);
    }
}

string
TRIPAttrHeader::getAttrFlagStr() const
{
    string retStr;
    if(_dataRep._flag & TRIP_FLG_OPT)
    {
        retStr += "|OPTIONAL";
    }
    if(_dataRep._flag & TRIP_FLG_TRANSITIVE)
    {
        retStr += "|TRANSITIVE";
    }
    if(_dataRep._flag & TRIP_FLG_DEPENDENT)
    {
        retStr += "|DEPENDENT";
    }
    if(_dataRep._flag & TRIP_FLG_PARTIAL)
    {
        retStr += "|PARTIAL";
    }
    if(_dataRep._flag & TRIP_FLG_LINK_STATE)
    {
        retStr += "|LINK-STATE";
    }
    retStr += "|";
    return retStr;
}

u_int8_t     
TRIPAttrHeader::getAttrFlag() const
{
    return (_dataRep._flag);
}

TRIPAttrTypeFlag 
TRIPAttrHeader::getAttrType() const
{
    return (static_cast<TRIPAttrTypeFlag>(_dataRep._type));
}

string           
TRIPAttrHeader::getAttrTypeStr() const
{
    string retStr;
    switch(static_cast<TRIPAttrTypeFlag>(_dataRep._type))
    {
        case TRIP_WITHDRAWN_ROUTE:
        {
            retStr = "WITHDRAWN_ROUTE";
        }
        break;
        case TRIP_REACHABLE_ROUTE:
        {
            retStr = "REACHABLE_ROUTE";
        }
        break;
        case TRIP_NEXT_HOP_SERVER:
        {
            retStr = "NEXT_HOP_SERVER";
        }
        break;
        case TRIP_ADVERTISEMENT_PATH:
        {
            retStr = "ADVERTISEMENT_PATH";
        }
        break;
        case TRIP_ROUTED_PATH:
        {
            retStr = "ROUTED_PATH";
        }
        break;
        case TRIP_ATOMIC_AGGREGATE:
        {
            retStr = "ATOMIC_AGGREGATE";
        }
        break;
        case TRIP_LOCAL_PREF:
        {
            retStr = "LOCAL_PREF";
        }
        break;
        case TRIP_MULTI_EXIT_DISC:
        {
            retStr = "MULTI_EXIT_DISC";
        }
        break;
        case TRIP_COMMUNITY:
        {
            retStr = "COMMUNITY";
        }
        break;
        case TRIP_ITAD_TOPOLOGY:
        {
            retStr = "ITAD_TOPOLOGY";
        }
        break;
        case TRIP_AUTHENTICATION:
        {
            retStr = "AUTHENTICATION";
        }
        break;
        default:
            retStr = "UNKNOWN";
        break;
    }
    return retStr;
}


TRIPAttrHeader::TRIPAttrHeader(const TRIPAttrHeader& other) 
{
    copyObj(other);    
}

TRIPAttrHeader& 
TRIPAttrHeader::operator=(const TRIPAttrHeader& other)
{
    if(this != &other)
    {
        copyObj(other);    
    }
    return (*this);
}

void
TRIPAttrHeader::copyObj(const TRIPAttrHeader& other)
{
    _dataRep._flag = other._dataRep._flag;
    _dataRep._type = other._dataRep._type;
    _dataRep._length = other._dataRep._length;
}

void 
TRIPAttrHeader::writeData(int id) const throw(VNetworkException&)
{
    TripAttrHeaderData sendData;
    sendData._flag = _dataRep._flag;
    sendData._type = _dataRep._type;
    sendData._length = htons(_dataRep._length);
    TRIPUtil::writeData(id, &sendData, 4);
}
