#ifndef TRIPAttrHeader_hxx
#define TRIPAttrHeader_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPAttrHeader_hxx_Version = 
"$Id: TRIPAttrHeader.hxx,v 1.3 2000/11/13 22:04:52 kevin Exp $";

#include <netinet/in.h>
#include <iostream.h>
#include <assert.h>

#include "global.h"
#include "vtypes.h"
#include "VNetworkException.hxx"
#include "TRIPBadDataException.hxx"

typedef enum {
    TRIP_FLG_OPT=1,
    TRIP_FLG_TRANSITIVE=(1 << 1),
    TRIP_FLG_DEPENDENT =(1 << 2),
    TRIP_FLG_PARTIAL=(1 << 3),
    TRIP_FLG_LINK_STATE=(1 << 4),
    TRIP_FLG_MAX = (1 << 5)
} TRIPAttrFlag;

typedef enum {
    TRIP_WITHDRAWN_ROUTE=1,
    TRIP_REACHABLE_ROUTE,
    TRIP_NEXT_HOP_SERVER,
    TRIP_ADVERTISEMENT_PATH,
    TRIP_ROUTED_PATH,
    TRIP_ATOMIC_AGGREGATE,
    TRIP_LOCAL_PREF,
    TRIP_MULTI_EXIT_DISC,
    TRIP_COMMUNITY,
    TRIP_ITAD_TOPOLOGY,
    TRIP_AUTHENTICATION,
    TRIP_ATTR_TYPE_MAX
} TRIPAttrTypeFlag;

typedef struct _TripHAttrTag {
        u_int8_t  _flag;
        u_int8_t  _type;
        u_int16_t _length;
} TripAttrHeaderData;

///TRIPAttrHeader

class TRIPAttrHeader {
    public:
        ///
        TRIPAttrHeader( u_int8_t aFlg, TRIPAttrTypeFlag type);

        ///
        virtual ~TRIPAttrHeader() { };

        //Accessors
        ///
        string           getAttrFlagStr() const;
        ///
        u_int8_t         getAttrFlag() const;
        ///
        TRIPAttrTypeFlag getAttrType() const;
        ///
        string           getAttrTypeStr() const;

        ///The return length of data only 
        u_int16_t getLength() const { return _dataRep._length; };

        ///The return length includes the length of the header also
        u_int16_t getAttrLength() const { return _dataRep._length+4; };

        ///Returns the fixed length of header
        virtual u_int16_t getHdrLength() { 
            return(sizeof(u_int32_t)); 
        }

        ///
        friend ostream& operator<<(ostream& os, const TRIPAttrHeader&);

        ///Writes object to given network socket in network byte order
        virtual void writeData(int id) const throw(VNetworkException&);

        ///
        virtual void dump(ostream& os) const { };

        ///
        TripAttrHeaderData& getDataRep() { return _dataRep; };

    protected:
        ///
        TRIPAttrHeader() { };
        ///
        TRIPAttrHeader(const char* buf) throw (TRIPBadDataException&);

        ///
        TRIPAttrHeader(const TRIPAttrHeader& other); 

        ///
        TRIPAttrHeader& operator=(const TRIPAttrHeader& other);

        ///
        void setLength(u_int16_t len) { _dataRep._length = len; };

        ///
        void copyObj(const TRIPAttrHeader& other);

        ///
        TripAttrHeaderData _dataRep;

        ///
        friend class TRIPMsg;
        friend class TRIPTransceiver;
        friend class TRIPMsgParser;
        friend class TRIPRouteAttr;
};

inline ostream& operator<<(ostream& os, const TRIPAttrHeader& obj)
{
    os << "**Attribute HEADER**" << endl;
    os.width(25);
    os << "Flag: " << obj.getAttrFlagStr() << endl;
    os.width(25);
    os << "Type: " << obj.getAttrTypeStr() << endl;
    os.width(25);
    os << "Length: " << static_cast<int>(obj._dataRep._length) << endl;
    obj.dump(os);
    return os;
}

#endif
