/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPAuthAttr_cxx_Version = 
"$Id: TRIPAuthAttr.cxx,v 1.2 2000/11/13 22:04:09 kevin Exp $";

#ifdef __linux__
    #include <algo.h>
#else
    #include <algorithm>
#endif

#include "TRIPAuthAttr.hxx"
#include "TRIPUtil.hxx"
#include "cpLog.h"


TRIPAuthAttr::TRIPAuthAttr(const TRIPAuthAttr& other) 
{
     copyObj(other);
}

TRIPAuthAttr& 
TRIPAuthAttr::operator=(const TRIPAuthAttr& other)
{
    if(this != &other)
    {
        copyObj(other);
    }
    return(*this);
}

void 
TRIPAuthAttr::dump(ostream& os) const
{
    os.width(25);
    os << "**AuthAttr**" << endl;
    os.width(25);
    for(TripAuthDataList::const_iterator itr = _authDataList.begin();
               itr != _authDataList.end(); itr++)
    {
        os << *(*itr) << endl;
    }
}

void 
TRIPAuthAttr::writeData(int id) const throw(VNetworkException&)
{
    TRIPRouteAttr::writeData(id);
    assert(_authDataList.size());
    for(TripAuthDataList::const_iterator itr = _authDataList.begin();
               itr != _authDataList.end(); itr++)
    {
        (*itr)->writeData(id);
    }
}

TRIPAuthAttr::TRIPAuthAttr(const char* buf) 
        throw (TRIPBadDataException&)
    : TRIPRouteAttr(buf)
{
    int hdrLen  = _hdr->getHdrLength(); 
    int len = _hdr->getLength();

    if(_hdr->getAttrType() != TRIP_AUTHENTICATION)
    {
        cpLog(LOG_ALERT, "Bad Authentication attribute");
        throw TRIPBadDataException("Bad Authentication attribute",
                                   __FILE__, __LINE__, 0);
    }

    u_int16_t tLen = 0;
    while(tLen < len)
    {
        Sptr<TRIPAuthData> aData = new TRIPAuthData(buf+hdrLen+tLen);
        tLen += aData->getLength();
        _authDataList.push_back(aData);
    }
}

void 
TRIPAuthAttr::copyObj(const TRIPAuthAttr& other)
{
    TRIPRouteAttr::copyObj(other);
    _authDataList.clear();
    for(TripAuthDataList::const_iterator itr = other._authDataList.begin();
               itr != other._authDataList.end(); itr++)
    {
        _authDataList.push_back(*itr);
    }
}

void 
TRIPAuthAttr::addAuthData(Sptr<TRIPAuthData> authData)
{
    _authDataList.push_back(authData);
    int len = getLength();
    setLength(len + authData->getLength());
}
