#ifndef TRIPCapability_hxx
#define TRIPCapability_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPCapability_hxx_Version = 
"$Id: TRIPCapability.hxx,v 1.3 2000/11/06 18:38:18 surendra Exp $";

#include <netinet/in.h>

#include "vtypes.h"
#include "VNetworkException.hxx"
#include "TRIPBadDataException.hxx"
#include "Sptr.hxx"
#include "TRIPData.hxx"

typedef enum {
    TRIP_CAP_ROUTE=1,
    TRIP_CAP_SND_REC,
    TRIP_CAP_MAX
} TRIPCap_Code;

///TRIPCapability

class TRIPCapability {
    public:
        ///
        TRIPCapability(u_int16_t code);

        ///
        virtual ~TRIPCapability() { };

        //Accessors

        ///
        u_int16_t getLength() const;

        ///
        u_int16_t getCode() const { return _code; };

        ///
        friend ostream& operator<<(ostream& os, const TRIPCapability&);

        ///
        virtual void dump(ostream& os) const =0;

        ///Writes object to given network socket in network byte order
        virtual void writeData(int id) const throw(VNetworkException&);

        ///
        virtual void validate() const throw (TRIPBadDataException&);

    protected:
        ///
        TRIPCapability() { };

        ///
        TRIPCapability(const char* buf) throw (TRIPBadDataException&);

        ///
        void copyObj(const TRIPCapability& other);

        ///
        u_int16_t _code;
        u_int16_t _length;

        ///
        friend class TRIPCapabilityInfo;
};

inline ostream& operator<<(ostream& os, const TRIPCapability& obj)
{
    os.width(25);
    os << endl << "**Capability**" << endl;
    os.width(25);
    os << "Code: " << static_cast<int>(obj._code) << endl;
    os.width(25);
    os << "length: " << static_cast<int>(obj._length) << endl;
    obj.dump(os);
    return os;
}

#endif
