#ifndef TRIPCapabilityInfo_hxx
#define TRIPCapabilityInfo_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPCapabilityInfo_hxx_Version = 
"$Id: TRIPCapabilityInfo.hxx,v 1.1 2000/10/26 19:51:09 surendra Exp $";

#include <netinet/in.h>
#include <vector>

#include "vtypes.h"
#include "VNetworkException.hxx"
#include "TRIPBadDataException.hxx"
#include "Sptr.hxx"
#include "TRIPParameter.hxx"
#include "TRIPRouteCapability.hxx"
#include "TRIPTransCapability.hxx"

///TRIPCapabilityInfo

class TRIPCapabilityInfo : public TRIPParameter {
    public:
        typedef vector<Sptr<TRIPCapability> >  CapabilityList;
        ///
        TRIPCapabilityInfo(): TRIPParameter(TRIP_PARAM_CAP)
        {
        };

        ~TRIPCapabilityInfo() { };

        ///
        TRIPCapabilityInfo(const TRIPCapabilityInfo& other); 

        ///
        TRIPCapabilityInfo& operator=(const TRIPCapabilityInfo& other);

        //Accessors
        ///
        const  CapabilityList& getData() { return _capabilityList; };

        //Mutator
        ///
        void addCapability(Sptr<TRIPCapability> data);

        ///
        void dump(ostream& os) const;

        ///Writes object to given network socket in network byte order
        void writeData(int id) const throw(VNetworkException&);

    private:
        ///
        void parseCapability(const char* buf, u_int16_t len);

        ///
        TRIPCapabilityInfo(const char* buf) throw (TRIPBadDataException&);

        ///
        void copyObj(const TRIPCapabilityInfo& other);

        ///
        CapabilityList  _capabilityList;

        ///
        friend class TRIPOpenMsg;
};

#endif
