/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "TRIPHeader.hxx"
#include "TRIPUtil.hxx"
#include "cpLog.h"

TRIPHeader::TRIPHeader(u_int8_t type, u_int16_t length)
{
    _dataRep._type = type;
    _dataRep._length = length;
}

TRIPHeader::TRIPHeader(const char* buf) throw(TRIPBadDataException&)
{
    u_int16_t len;
    u_int8_t  type;
    memcpy(&len, buf, 2);
    memcpy(&type, buf+2, 1);
    _dataRep._length = ntohs(len); 
    _dataRep._type = type; 

    if((_dataRep._type < 1) || (_dataRep._type >= TRIP_OP_MAX))
    {
        cpLog(LOG_ALERT, "BAD TRIP data");
        throw TRIPBadDataException("BAD TRIP Data", __FILE__, __LINE__, 0);
    }
}

TRIPHeader::TRIPHeader(const TRIPHeader& other) 
{
    copyObj(other);    
}

TRIPHeader& 
TRIPHeader::operator=(const TRIPHeader& other)
{
    if(this != &other)
    {
        copyObj(other);    
    }
    return (*this);
}

void
TRIPHeader::copyObj(const TRIPHeader& other)
{
    _dataRep._length = other._dataRep._length;
    _dataRep._type = other._dataRep._type;
}

void 
TRIPHeader::setType(u_int8_t type)
{
    assert((type > 1) && (type < TRIP_OP_MAX));
    _dataRep._type = type;
}
        
void 
TRIPHeader::writeData(int id) const throw(VNetworkException&)
{
    TripHeaderData sendData;
    sendData._length = htons(_dataRep._length);
    sendData._type = _dataRep._type;
    TRIPUtil::writeData(id, &sendData, 3);
}
