#ifndef TRIPHeader_hxx
#define TRIPHeader_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPHeader_hxx_Version = 
"$Id: TRIPHeader.hxx,v 1.4 2000/11/13 21:57:16 kevin Exp $";

#include <netinet/in.h>
#include <iostream.h>
#include <assert.h>

#include "global.h"
#include "vtypes.h"
#include "VNetworkException.hxx"
#include "TRIPBadDataException.hxx"

typedef enum {
    TRIP_OP_OPEN=1,
    TRIP_OP_UPDATE,
    TRIP_OP_NOTIF,
    TRIP_OP_KEEPALIVE,
    TRIP_OP_MAX
} TRIPOpCode;

typedef struct _TripHTag {
        u_int16_t _length;
        u_int8_t  _type;
} TripHeaderData;

///TRIPHeader

class TRIPHeader {
    public:
        ///
        TRIPHeader(u_int8_t type, u_int16_t length);

        ///
        TRIPHeader(const char* buf) throw (TRIPBadDataException&);

        ///
        TRIPHeader(const TRIPHeader& other); 

        ///
        TRIPHeader& operator=(const TRIPHeader& other);

        //Accessors
        ///
        bool isAOpenHdr() const { return (_dataRep._type == TRIP_OP_OPEN); };

        ///
        bool isAUpdateHdr() const { return (_dataRep._type == TRIP_OP_UPDATE);};

        ///
        bool isANotificationHdr() const { 
            return (_dataRep._type == TRIP_OP_NOTIF); 
        };

        ///
        bool isAKeepAliveHdr() const { 
            return (_dataRep._type == TRIP_OP_KEEPALIVE); 
        };

        ///The return length includes the length of the header also
        u_int16_t getMsgLength() const { return _dataRep._length; };

        ///Returns the fixed length of header
        static u_int16_t getHdrLength() { 
            return(sizeof(u_int16_t)+sizeof(u_int8_t)); 
        }

        //Mutators
        //Set the header type
        void setType(u_int8_t type);
        
        ///
        friend ostream& operator<<(ostream& os, const TRIPHeader&);

        ///Writes object to given network socket in network byte order
        void writeData(int id) const throw(VNetworkException&);

        ///
        TripHeaderData& getDataRep() { return _dataRep; };

        ///
        u_int8_t getType() const { return _dataRep._type; };

    private:

        ///
        void setMsgLength(u_int16_t len) { _dataRep._length = len; };

        ///
        void copyObj(const TRIPHeader& other);

        ///
        TripHeaderData _dataRep;

        ///
        friend class TRIPMsg;
        friend class TRIPTransceiver;
        friend class TRIPMsgParser;
};

inline ostream& operator<<(ostream& os, const TRIPHeader& obj)
{
    os.width(25);
    os << "**MSG HEADER**" << endl;
    os.width(25);
    os << "Type: " << static_cast<int>(obj._dataRep._type) << endl;
    os.width(25);
    os << "length: " << static_cast<int>(obj._dataRep._length) << endl;
    return os;
}


#endif
