/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include <assert.h>

#include "TRIPLinkStateAttrHeader.hxx"
#include "TRIPUtil.hxx"
#include "cpLog.h"

TRIPLinkStateAttrHeader::TRIPLinkStateAttrHeader(u_int8_t aFlg, TRIPAttrTypeFlag type,
                        const TRIPId& id, u_int32_t seqNum)
     : TRIPAttrHeader(aFlg, type) ,
       _tripId(new TRIPId(id)),
       _seqNum(seqNum)
{   
     assert(aFlg & TRIP_FLG_LINK_STATE);
};


TRIPLinkStateAttrHeader::TRIPLinkStateAttrHeader(const char* buf) throw(TRIPBadDataException&)
    : TRIPAttrHeader(buf)
{
    _tripId = new TRIPId(buf+4);
    memcpy(&_seqNum, buf+8, 4);
    _seqNum = ntohl(_seqNum);
}

TRIPLinkStateAttrHeader::TRIPLinkStateAttrHeader(const TRIPLinkStateAttrHeader& other) 
{
    copyObj(other);    
}

TRIPLinkStateAttrHeader& 
TRIPLinkStateAttrHeader::operator=(const TRIPLinkStateAttrHeader& other)
{
    if(this != &other)
    {
        copyObj(other);    
    }
    return (*this);
}

void
TRIPLinkStateAttrHeader::copyObj(const TRIPLinkStateAttrHeader& other)
{
    TRIPAttrHeader::copyObj(other);
    _tripId = other._tripId;
    _seqNum = other._seqNum;
}

void 
TRIPLinkStateAttrHeader::writeData(int id) const throw(VNetworkException&)
{
    TRIPAttrHeader::writeData(id);
    _tripId->writeData(id);
    u_int32_t seqNum = htonl(_seqNum);
    TRIPUtil::writeData(id, &seqNum, 4);
}

void
TRIPLinkStateAttrHeader::dump(ostream& os) const
{
    os.width(25);
    os << "TRIP Id:" << *_tripId << endl;
    os.width(25);
    os << "Seq Number:" << _seqNum << endl;
}
