#ifndef TRIPMsg_hxx
#define TRIPMsg_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const TRIPMsg_hxx_Version = 
"$Id: TRIPMsg.hxx,v 1.3 2000/11/06 18:40:25 surendra Exp $";

#include "Sptr.hxx"
#include "TRIPHeader.hxx"
#include "TRIPBadDataException.hxx"

///TRIPMsg

class TRIPMsg {
    public:
       ///
       const TRIPHeader& getHeader() const { return *_hdr; };

       ///
       virtual ~TRIPMsg() { };

       //Accessors
       ///
       virtual bool isAOpenMsg() const { return false; };

       ///
       virtual bool isAUpdateMsg() const { return false; };

       ///
       virtual bool isANotificationMsg() const { return false; }; 

       ///
       virtual bool isAKeepAliveMsg() const { return false; }; 

       ///
       virtual TRIPOpCode getType() const =0;

       ///
       u_int16_t getMsgLength() const { return (_hdr->getMsgLength()); };

       ///
       friend ostream& operator<<(ostream& os, const TRIPMsg&);

       ///Checks the sanity of TRIP Msg, and throws a Bad Data exception
       virtual void checkSanity() const throw (TRIPBadDataException&) = 0;

       ///Writes object to given network socket in network byte order
       virtual void writeData(int id) const throw(VNetworkException&) = 0;

    protected:
       ///
       virtual void parse(const TRIPHeader& hdr,
                          const char* data) throw (TRIPBadDataException&) = 0;
       ///
       virtual void parse(const char* data) throw (TRIPBadDataException&) = 0;

       ///
       virtual void dump(ostream& os)const = 0;


       ///
       virtual string className() const= 0;

       ///
       void parseHeader(const char* data) throw (TRIPBadDataException&);

       ///
       TRIPMsg() : _hdr(0) { };

       ///Sets the message length.The length should also include header len
       void setMsgLength(u_int16_t len) const { _hdr->setMsgLength(len); };


       ///
       Sptr<TRIPHeader> _hdr;

       ///
       friend class TRIPMsgParser;
};

inline ostream& operator<<(ostream& os, const TRIPMsg& obj)
{
    obj.dump(os);
    return os;
}

#endif
