/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPNextHopServer_cxx_Version = 
"$Id: TRIPNextHopServer.cxx,v 1.2 2000/11/02 02:12:11 surendra Exp $";

#include "TRIPNextHopServer.hxx"
#include "TRIPUtil.hxx"
#include "cpLog.h"


TRIPNextHopServer::TRIPNextHopServer(const TRIPNextHopServer& other) 
{
     copyObj(other);
}

TRIPNextHopServer& 
TRIPNextHopServer::operator=(const TRIPNextHopServer& other)
{
    if(this != &other)
    {
        copyObj(other);
    }
    return(*this);
}

void 
TRIPNextHopServer::dump(ostream& os) const
{
    os.width(25);
    os << "**NextHopServer**" << endl;
    os.width(25);
    os << "Itad:" << static_cast<int>(_itad) << endl;
    os.width(25);
    os << "Length:" << static_cast<int>(_length) << endl;
    os.width(25);
    os << "Address:" << *_data << endl;
}

void 
TRIPNextHopServer::writeData(int id) const throw(VNetworkException&)
{
    TRIPRouteAttr::writeData(id);

    struct shipIt {
        u_int16_t  itad;
        u_int16_t  len;
    } sIt;

    sIt.itad = htons(_itad);
    sIt.len = htons(_length);
    TRIPUtil::writeData(id, &sIt, sizeof(struct shipIt));
    TRIPUtil::writeData(id, _data->getData(), _data->length());
}

TRIPNextHopServer::TRIPNextHopServer(const char* buf) 
        throw (TRIPBadDataException&)
    : TRIPRouteAttr(buf)
{
    struct shipIt {
        u_int16_t  itad;
        u_int16_t  len;
    } sIt;

    int hdrLen  = _hdr->getHdrLength(); 

    if(_hdr->getAttrType() != TRIP_NEXT_HOP_SERVER)
    {
        cpLog(LOG_ALERT, "Bad Advertise path");
        throw TRIPBadDataException("Bad Advertise path",
                                   __FILE__, __LINE__, 0);
    }

    memcpy(&sIt, buf+hdrLen, sizeof(struct shipIt));
    _itad = ntohs(sIt.itad);
    _length = ntohs(sIt.len);

    //Extract data past attrHeader+4bytes(includes itad and data length)
    _data = new TRIPData(buf+hdrLen+4, _length);
}

void 
TRIPNextHopServer::copyObj(const TRIPNextHopServer& other)
{
    TRIPRouteAttr::copyObj(other);
    _itad = other._itad;
    _length = other._length;
    _data = other._data;
}
