/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const TRIPOpenMsg_cxx_Version = 
"$Id: TRIPOpenMsg.cxx,v 1.4 2000/11/09 02:35:52 surendra Exp $";

#include <strstream.h>

#include "TRIPOpenMsg.hxx"
#include "TRIPUtil.hxx"
#include "cpLog.h"

TRIPOpenMsg::TRIPOpenMsg(u_int16_t itadNumber, 
                         u_int32_t tripId,u_int16_t holdTime)
    :TRIPMsg(), 
     _version(1), 
     _res(0),
     _itad(itadNumber), 
     _tripId(tripId), 
     _holdTime(holdTime),  
     _paramLen(0)
{
    //Length of message without any parameters is 15 bytes
    _hdr = new TRIPHeader(TRIP_OP_OPEN, 15);
}

void 
TRIPOpenMsg::dump(ostream& os)const
{
   os.width(25);
   os << "**OpenMsg**" << endl;
   os.width(25);
   os << *_hdr << endl;
   os.width(25);
   os << "Version:" << static_cast<int>(_version) << endl;
   os.width(25);
   os << "ITAD:" << static_cast<int>(_itad) << endl;
   os.width(25);
   os << "TRIPID:" << _tripId << endl;
   os.width(25);
   os << "HoldTime(sec):" << _holdTime << endl;
   os.width(25);
   os << "Param length:" << _paramLen << endl;
   os.width(25);
   os << "Optional Parameters:" << endl;
   ParameterList::const_iterator itr;
   for(itr = _parameterList.begin(); 
             itr != _parameterList.end(); itr++)
   {
       const Sptr<TRIPParameter>& param = (*itr);
       os << endl << *param << endl;
   }
}

void 
TRIPOpenMsg::parse(const TRIPHeader& hdr,
                          const char* data) throw (TRIPBadDataException&)
{
    //char buf[256];
    //ostrstream oStr(buf, 256);
    //oStr << hdr << endl << ends;
    //cpLog(LOG_DEBUG, "%s", oStr.str());

    if((!hdr.isAOpenHdr()) || (hdr.getMsgLength() < 15))
    {
        cpLog(LOG_ALERT, "Bad Open Msg");
        throw TRIPBadDataException("Bad Open Msg", __FILE__, __LINE__, 0);
    }

    _hdr = new TRIPHeader(hdr);

    struct shipIt {
        u_int8_t    version;
        u_int8_t    res;
        u_int16_t   itad;
        u_int32_t   tripId;
        u_int16_t   hTime;
        u_int16_t   pLen;
    } sIt;

 
    int idx = 0;
    memcpy(&sIt, data , sizeof(struct shipIt)); 
    idx += sizeof(struct shipIt);
    _version = sIt.version;
    _res = sIt.res;
    _itad = ntohs(sIt.itad);
    _tripId = TRIPId(ntohl(sIt.tripId));
    _holdTime = ntohs(sIt.hTime);
    _paramLen = ntohs(sIt.pLen);
    int pLen = _paramLen;

    u_int16_t tLen = 0;
    while(pLen && tLen < pLen)
    {
        u_int16_t type;
        u_int16_t len;
        memcpy(&type, data+idx, 2);
        memcpy(&len, data+idx+2, 2);
        type = ntohs(type);
        len = ntohs(len);
        switch(static_cast<TRIPParamType>(type))
        {
            case TRIP_PARAM_CAP:
            {
                Sptr<TRIPCapabilityInfo> cInfo = new TRIPCapabilityInfo(data+idx); 
                addParameter(cInfo);
                idx += cInfo->getParamLength();
                tLen += cInfo->getLength();
                assert(len == cInfo->getParamLength());
            }
            break;
            default:
                cpLog(LOG_DEBUG, "Unsupported param type (%d)", type);
                tLen += len;
                break;
            break;
        }  
    }
}

void 
TRIPOpenMsg::parse(const char* data) throw (TRIPBadDataException&) 
{
    TRIPHeader hdr(data);
    parse(hdr, data+3);
}


void 
TRIPOpenMsg::writeData(int id) const throw(VNetworkException&)
{
   TRIPMsg::writeData(id);

   struct shipIt {
       u_int8_t    version;
       u_int8_t    res;
       u_int16_t   itad;
       u_int32_t   tripId;
       u_int16_t   hTime;
       u_int16_t   pLen;
   } sIt;
   sIt.version = _version;
   sIt.res = _res;
   sIt.itad = htons(_itad);
   sIt.tripId = htonl(_tripId.getId());
   sIt.hTime = htons(_holdTime);
   sIt.pLen = htons(_paramLen);

   TRIPUtil::writeData(id, &sIt, sizeof(struct shipIt));

   ParameterList::const_iterator itr;
   for(itr = _parameterList.begin(); 
             itr != _parameterList.end(); itr++)
   {
       const Sptr<TRIPParameter>& param = (*itr);
       param->writeData(id);
   }
}

void
TRIPOpenMsg::addParameter(Sptr<TRIPParameter> parameter)
{
    _parameterList.push_back(parameter);
    _paramLen += parameter->getLength();
    setMsgLength(_paramLen + 15);
}


void 
TRIPOpenMsg::checkSanity() const throw (TRIPBadDataException&)
{
    if(!(_hdr->isAOpenHdr() && (_hdr->getMsgLength() < 15)))
    {
        cpLog(LOG_ALERT, "Bad Open Msg");
        throw TRIPBadDataException("Bad Open Msg", __FILE__, __LINE__, 0);
    }
}
