#ifndef TRIPOpenMsg_hxx
#define TRIPOpenMsg_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const TRIPOpenMsg_hxx_Version = 
"$Id: TRIPOpenMsg.hxx,v 1.4 2000/11/09 00:30:39 surendra Exp $";

#include <vector>

#include "TRIPMsg.hxx"
#include "TRIPId.hxx"
#include "TRIPCapabilityInfo.hxx"

///TRIPOpenMsg

class TRIPOpenMsg : public TRIPMsg
{
    public:
       ///
       typedef vector<Sptr<TRIPParameter> > ParameterList;

       ///Default hold time is 240 seconds
       TRIPOpenMsg(u_int16_t itadNumber, u_int32_t tripId,u_int16_t holdTime=240);

       ///
       bool isAOpenMsg() const { return true; }; 

       ///
       TRIPOpCode getType() const  { return TRIP_OP_OPEN; };

       ///
       u_int16_t getHoldTime() const { return _holdTime; };


       ///
       virtual void dump(ostream& os)const;


       ///Writes object to given network socket in network byte order
       void writeData(int id) const throw(VNetworkException&);

       ///
       string className() const { return "TRIPOpenMsg"; };

       ///Checks the sanity of TRIP Msg, and throws a Bad Data exception
       void checkSanity() const throw (TRIPBadDataException&);

       ///
       const ParameterList& getParameters() const { return _parameterList; }

       ///
       void addParameter(Sptr<TRIPParameter> parameter);

       ///
       void setHoldTimer(u_int16_t hTime) { _holdTime = hTime; };

       ///
       u_int16_t  getItad() const { return _itad; };

       ///
       const TRIPId&  getTripId() const { return _tripId; };

    private:
       TRIPOpenMsg()  { }; 
       ///
       void parse(const TRIPHeader& hdr,
                          const char* data) throw (TRIPBadDataException&);

       ///
       void parse(const char* data) throw (TRIPBadDataException&);


       u_int8_t    _version;
       u_int8_t    _res;
       u_int16_t   _itad;
       TRIPId      _tripId;
       u_int16_t   _holdTime;
       u_int16_t   _paramLen;
       ParameterList _parameterList;

       ///
       friend TRIPMsgParser;
};

#endif
