#ifndef TRIPParameter_hxx
#define TRIPParameter_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPParameter_hxx_Version = 
"$Id: TRIPParameter.hxx,v 1.3 2000/11/13 21:52:23 kevin Exp $";

#include <netinet/in.h>
#include <iostream.h>
#include <assert.h>

#include "global.h"
#include "vtypes.h"
#include "VNetworkException.hxx"
#include "TRIPBadDataException.hxx"

typedef enum {
   TRIP_PARAM_CAP=1,
   TRIP_PARAM_MAX
} TRIPParamType;

///TRIPParameter

class TRIPParameter {
    public:

        ///
        virtual ~TRIPParameter() { };

        //Accessors

        ///Length of parameter header + parameter length
        u_int16_t getLength() const { return (4+_paramLength); };

        ///
        u_int16_t getParamLength() const { return _paramLength; };

        ///
        u_int16_t getType() const { return _type; };

        ///
        friend ostream& operator<<(ostream& os, const TRIPParameter&);

        ///
        virtual void dump(ostream& os) const = 0;

        ///Writes object to given network socket in network byte order
        virtual void writeData(int id) const throw(VNetworkException&);

    protected:
        ///
        TRIPParameter() { };

        ///
        TRIPParameter(u_int16_t type): _type(type), _paramLength(0) 
        {
        };

        ///
        TRIPParameter(const TRIPParameter& other)
        {
            _type = other._type;
            _paramLength = other._paramLength;
        }

        ///
        TRIPParameter(const char* buf) throw (TRIPBadDataException&);

        ///
        u_int16_t _type;
        u_int16_t _paramLength;

        ///
        friend class TRIPOpenMsg;
};

inline ostream& operator<<(ostream& os, const TRIPParameter& obj)
{
    os.width(25);
    os << "**Parameter**" << endl;
    os.width(25);
    os << "Type:" << static_cast<int>(obj._type) << endl;
    os.width(25);
    os << "Length:" << static_cast<int>(obj._paramLength) << endl;
    obj.dump(os);
    return os;
}

#endif
