/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPPathContainer_cxx_Version = 
"$Id: TRIPPathContainer.cxx,v 1.3 2000/11/08 22:57:57 surendra Exp $";

#include "TRIPPathContainer.hxx"
#include "TRIPItadPathSegment.hxx"
#include "cpLog.h"


TRIPPathContainer::TRIPPathContainer(const TRIPPathContainer& other) 
{
     copyObj(other);
}

TRIPPathContainer& 
TRIPPathContainer::operator=(const TRIPPathContainer& other)
{
    if(this != &other)
    {
        copyObj(other);
    }
    return(*this);
}

void 
TRIPPathContainer::addPathSegment(const TRIPItadPathSegment& data)
{
    _segmentList.push_back(data);
    _len += data.getLength();
    setLength(_len);
}

TRIPPathContainer&
TRIPPathContainer::operator<<(const TRIPItadPathSegment& data)
{
    addPathSegment(data);
    return *this;
}

void 
TRIPPathContainer::dump(ostream& os) const
{
    os.width(25);
    PathSegmentList::const_iterator itr;
    for(itr = _segmentList.begin(); 
              itr != _segmentList.end(); itr++)
    {
         const TRIPItadPathSegment& data = (*itr);
         os << data << endl;
    }
}

void 
TRIPPathContainer::writeData(int id) const throw(VNetworkException&)
{
    TRIPRouteAttr::writeData(id);
    PathSegmentList::const_iterator itr;
    for(itr = _segmentList.begin(); 
              itr != _segmentList.end(); itr++)
    {
         const TRIPItadPathSegment& data = (*itr);
         data.writeData(id);
    }
}

TRIPPathContainer::TRIPPathContainer(const char* buf) 
        throw (TRIPBadDataException&)
    : TRIPRouteAttr(buf)
{
    int hdrLen  = _hdr->getHdrLength(); 
    int len  = _hdr->getLength(); 

    _len = 0;
    parsePathSegments(buf+hdrLen, len);
}

void
TRIPPathContainer::parsePathSegments(const char* buf, u_int16_t len)
{
    u_int16_t tLen = 0;
    while(tLen < len)
    {
        TRIPItadPathSegment tsData(buf+tLen);                
        tLen += tsData.getLength();
        addPathSegment(tsData);
    }
}

void 
TRIPPathContainer::copyObj(const TRIPPathContainer& other)
{
    TRIPRouteAttr::copyObj(other);
    _segmentList.clear();
    _len = 0;
    PathSegmentList::const_iterator itr;
    for(itr = other._segmentList.begin(); 
              itr != other._segmentList.end(); itr++)
    {
         const TRIPItadPathSegment& data = (*itr);
         addPathSegment(data);
    }
}
