#ifndef TRIPRouteAttr_hxx
#define TRIPRouteAttr_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPRouteAttr_hxx_Version = 
"$Id: TRIPRouteAttr.hxx,v 1.2 2000/11/03 01:02:13 surendra Exp $";

#include "VNetworkException.hxx"
#include "TRIPAttrHeader.hxx"
#include "TRIPLinkStateAttrHeader.hxx"

///TRIPRouteAttr

class TRIPRouteAttr {
    public:
        ///
        virtual ~TRIPRouteAttr() { };

        //Accessors

        ///
        u_int16_t getLength() const 
        { 
            return (_hdr->getHdrLength() + _hdr->getLength());
        };

        const TRIPAttrHeader& getHeader() const { return *_hdr; };

        ///
        friend ostream& operator<<(ostream& os, const TRIPRouteAttr&);

        ///
        virtual void dump(ostream& os) const = 0;

        ///Writes object to given network socket in network byte order
        virtual void writeData(int id) const throw(VNetworkException&);

    protected:
        ///
        TRIPRouteAttr(): _hdr(0) { };

        ///Creates a normal Atribute
        TRIPRouteAttr(u_int8_t aFlg, TRIPAttrTypeFlag type) ;

        ///Creates a link-state attribute
        TRIPRouteAttr(u_int8_t aFlg, 
                      TRIPAttrTypeFlag type, const TRIPId& id, u_int32_t seqNum);

        ///
        void setLength(u_int16_t len) { _hdr->setLength(len); };

        ///
        void copyObj(const TRIPRouteAttr& other)
        {
            _hdr = other._hdr;
        }

        ///
        TRIPRouteAttr(const char* buf) throw (TRIPBadDataException&);

        ///
        Sptr<TRIPAttrHeader> _hdr;

        ///
        friend class TRIPUpdateMsg;
};

inline ostream& operator<<(ostream& os, const TRIPRouteAttr& obj)
{
    os.width(25);
    os << "**Route Attribute**" << endl;
    os.width(25);
    os << *(obj._hdr) << endl;
    obj.dump(os);
    return os;
}

#endif
