/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

#include "TRIPRouteCapability.hxx"
#include "TRIPUtil.hxx"
#include "cpLog.h"

TRIPRouteCapability::TRIPRouteCapability(const char* buf) 
            throw(TRIPBadDataException&)
    :TRIPCapability(buf)
{
    if((_length % 4) != 0)
    {
        cpLog(LOG_ALERT, "BAD route capability data");
        throw TRIPBadDataException("BAD route capability data",
                                   __FILE__, __LINE__, 0);
    }
    int cLen = 0;
    int len = _length;
    //Reset the _length to minimum value as it will be changed as 
    //Routes are added
    _length = 0;
    while(cLen < len)
    {
        Sptr<TRIPRouteHdr> rHdr = new TRIPRouteHdr(buf+4+cLen);
        addRouteCapability(rHdr);
        cLen += rHdr->getLength();
    }
}

void
TRIPRouteCapability::addRouteCapability(Sptr<TRIPRouteHdr> rHdr)
{
    _routeList.push_back(rHdr);
    _length += rHdr->getLength();
}

TRIPRouteCapability::TRIPRouteCapability(const TRIPRouteCapability& other) 
{
    copyObj(other);    
}

TRIPRouteCapability& 
TRIPRouteCapability::operator=(const TRIPRouteCapability& other)
{
    if(this != &other)
    {
        copyObj(other);    
    }
    return (*this);
}

void
TRIPRouteCapability::copyObj(const TRIPRouteCapability& other)
{
    TRIPCapability::copyObj(other);
    _length = 0;
    _routeList.clear();

    RouteList::const_iterator itr;
    for(itr = other._routeList.begin(); 
              itr != other._routeList.end(); itr++)
    {
        _routeList.push_back(*itr);
    }
    _length = _routeList.size() * 4;
}

void 
TRIPRouteCapability::writeData(int id) const throw(VNetworkException&)
{
    TRIPCapability::writeData(id);
    RouteList::const_iterator itr;
    for(itr = _routeList.begin(); 
              itr != _routeList.end(); itr++)
    {
        (*itr)->writeData(id);
    }
}

void 
TRIPRouteCapability::dump(ostream& os) const
{
    os.width(25);
    os << "**RouteCapability**" << endl;
    RouteList::const_iterator itr;
    for(itr = _routeList.begin(); 
              itr != _routeList.end(); itr++)
    {
        os.width(25);
        os << *(*itr) << endl;;
    }
}

