#ifndef TRIPRouteContainer_hxx
#define TRIPRouteContainer_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPRouteContainer_hxx_Version = 
"$Id: TRIPRouteContainer.hxx,v 1.3 2000/11/08 22:58:04 surendra Exp $";

#include <netinet/in.h>
#include <vector>

#include "vtypes.h"
#include "VNetworkException.hxx"
#include "TRIPBadDataException.hxx"
#include "TRIPRouteAttr.hxx"
#include "TRIPRoutes.hxx"

///TRIPRouteContainer

class TRIPRouteContainer : public TRIPRouteAttr {
    public:
        ///
        typedef vector<TRIPRoutes > RouteList;

        virtual ~TRIPRouteContainer() { };

        ///
        TRIPRouteContainer(const TRIPRouteContainer& other); 

        ///
        TRIPRouteContainer& operator=(const TRIPRouteContainer& other);

        //Accessors
        ///
        const  RouteList& getRouteList() const { return _routeList; };

        //Mutator
        ///
        TRIPRouteContainer& operator<<(const TRIPRoutes& data);

        ///
        virtual void dump(ostream& os) const;

        ///Writes object to given network socket in network byte order
        virtual void writeData(int id) const throw(VNetworkException&);

    protected:
        ///
        void addRoute(const TRIPRoutes& data);

        ///Create a normal attribute
        TRIPRouteContainer(u_int8_t flg, TRIPAttrTypeFlag type)
            : TRIPRouteAttr(flg, type), _len(0)
        {
        };

        ///Create a link-state attribute
        TRIPRouteContainer(u_int8_t flg, TRIPAttrTypeFlag type,
                          const TRIPId& id, u_int32_t seqNum)
            : TRIPRouteAttr(flg, type, id, seqNum), _len(0)
        {
        };
        ///
        void parseRoutes(const char* buf, u_int16_t len);

        ///
        TRIPRouteContainer(const char* buf) throw (TRIPBadDataException&);

        ///
        void copyObj(const TRIPRouteContainer& other);

        ///
        RouteList  _routeList;

        ///
        u_int16_t        _len;

        ///
        friend class TRIPUpdateMsg;
};

#endif
