#ifndef TRIPRouteHdr_hxx
#define TRIPRouteHdr_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const TRIPRouteHdr_hxx_Version = 
"$Id: TRIPRouteHdr.hxx,v 1.3 2000/11/13 21:51:35 kevin Exp $";

#include <netinet/in.h>
#include <iostream.h>
#include <assert.h>

#include "global.h"
#include "vtypes.h"
#include "VNetworkException.hxx"
#include "TRIPBadDataException.hxx"

typedef enum {
    TRIP_ADDR_POTS=1,
    TRIP_ADDR_ROUTING_NUM,
    TRIP_ADDR_MAX
} TRIPAddrFamily;

typedef enum {
    TRIP_APP_PROTO_SIP=1,
    TRIP_APP_PROTO_H323_Q921,
    TRIP_APP_PROTO_H323_RAS,
    TRIP_APP_PROTO_H323_ANNEX_G,
    TRIP_APP_PROTO_MAX
} TRIPAppProto;


///TRIPRouteHdr

class TRIPRouteHdr {
    public:
        ///
        TRIPRouteHdr(TRIPAddrFamily addrFamily, TRIPAppProto appProtocol);

        ///
        TRIPRouteHdr(const char* buf) throw (TRIPBadDataException&);

        ///
        TRIPRouteHdr(const TRIPRouteHdr& other); 

        ///
        TRIPRouteHdr& operator=(const TRIPRouteHdr& other);

        //Accessors
        ///
        TRIPAddrFamily getAddrFamily() const { return _addrFamily; }
        ///
        TRIPAppProto getAppProtocol() const { return _appProto; }
        ///
        u_int16_t getLength() const { return sizeof(u_int32_t); };

        ///
        friend ostream& operator<<(ostream& os, const TRIPRouteHdr&);

        ///Writes object to given network socket in network byte order
        void writeData(int id) const throw(VNetworkException&);

    private:
        TRIPAddrFamily  _addrFamily;
        TRIPAppProto    _appProto;
};

inline ostream& operator<<(ostream& os, const TRIPRouteHdr& obj)
{
    os << "(" << static_cast<int>(obj._addrFamily) << ", ";
    os << static_cast<int>(obj._appProto) << ")" ;
    return os;
}

#endif
