
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const TRIPTransceiver_cxx_Version = 
"$Id: TRIPTransceiver.cxx,v 1.3 2000/11/09 02:35:54 surendra Exp $";
#include <strstream.h>

#include <strstream.h>

#include "TRIPTransceiver.hxx"
#include "TRIPMsgParser.hxx"
#include "cpLog.h"

void 
TRIPTransceiver::sendMsg(TRIPMsg& msg, int fd) throw (VNetworkException&, TRIPBadDataException&)
{
    msg.checkSanity();
    msg.writeData(fd);
}

Sptr<TRIPMsg> 
TRIPTransceiver::receiveMsg(int fd) throw (VNetworkException&, TRIPBadDataException& )
{
    int nread =0;
    char hBuf[3];

    TRIPUtil::readData(fd, (void*)&hBuf,  3, nread);
    if(nread == 0)
    {
        cpLog(LOG_ALERT, "Error reading connection");
        throw VNetworkException("Error reading connection",__FILE__, __LINE__, 0);
    }

    if(nread != 3)
    {
        throw TRIPBadDataException("Bad TRIP message",__FILE__, __LINE__, 0);
    }

    TRIPHeader hdr(hBuf);
    char lbuf[256];
    ostrstream lStr(lbuf, 256);
    lStr << "Header read:" << endl << hdr << endl << ends; 
    cpLog(LOG_DEBUG, "%s", lStr.str());

    int dataLen = hdr.getMsgLength() - hdr.getHdrLength();
    cpLog(LOG_DEBUG, "TRIP Msg length :[%d]\n",dataLen );
    char* buf = new char[dataLen+1];
    nread = 0;

    TRIPUtil::readData(fd, buf,  dataLen, nread);
    buf[dataLen] = '\0';
    cpLog(LOG_DEBUG, "Data read length:[%d]\n" ,nread );

    if(nread != dataLen)
    {
        throw TRIPBadDataException("Bad TRIP message",__FILE__, __LINE__, 0);
    }

    TRIPMsgParser prser;
    Sptr<TRIPMsg> msg = prser.parse(hdr, buf);
    delete [] buf;
    char mbuf[8096];
    ostrstream oStr(mbuf, 8095);
    oStr << *msg << endl;
    oStr << ends;
    cpLog(LOG_DEBUG, "Message received:\n%s", oStr.str());
    return msg;
}
