/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const TRIPUpdateMsg_cxx_Version = 
"$Id: TRIPUpdateMsg.cxx,v 1.5 2000/11/10 01:05:54 surendra Exp $";

#include "TRIPUpdateMsg.hxx"
#include "TRIPUtil.hxx"
#include "cpLog.h"

TRIPUpdateMsg::TRIPUpdateMsg()
    :TRIPMsg() 
{
    //Length of message without any parameters is 15 bytes
    _hdr = new TRIPHeader(TRIP_OP_UPDATE, 3);
}

void 
TRIPUpdateMsg::dump(ostream& os)const
{
   os.width(25);
   os << "**UpdateMsg**" << endl;
   checkSanity();
   os.width(25);
   os << *_hdr << endl;
   os.width(25);
   os << *_withdrawnRoute << endl;
   os.width(25);
   os << *_reachableRoute << endl;
   os.width(25);
   os << *_nextHopServer << endl;
   os.width(25);
   os << *_advertisePath << endl;
   os.width(25);
   os << *_routedPath << endl;
   if(_atomicAggregate != 0)
   {
       os.width(25);
       os << *_atomicAggregate << endl;
   }
   if(_localPref != 0)
   {
       os.width(25);
       os << *_localPref << endl;
   }
   if(_community != 0)
   {
       os.width(25);
       os << *_community << endl;
   }
   if(_multiExitDisc != 0)
   {
       os.width(25);
       os << *_multiExitDisc << endl;
   }
   if(_itadTopology != 0)
   {
       os.width(25);
       os << *_itadTopology << endl;
   }
   if(_authAttr != 0)
   {
       os.width(25);
       os << *_authAttr << endl;
   }
}

void 
TRIPUpdateMsg::parse(const TRIPHeader& hdr,
                          const char* data) throw (TRIPBadDataException&)
{
    if(!(hdr.isAUpdateHdr()))
    {
        cpLog(LOG_ALERT, "Bad Update Msg");
        throw TRIPBadDataException("Bad Update Msg", __FILE__, __LINE__, 0);
    }

    _hdr = new TRIPHeader(hdr);

    int tLen = hdr.getMsgLength() - hdr.getHdrLength();
    int cLen = 0;
    while(cLen < tLen)
    {
        TripAttrHeaderData hData;
        memcpy(&hData, data+cLen, sizeof(TripAttrHeaderData));
        u_int8_t type = hData._type;
        u_int16_t len = ntohs(hData._length);
        switch(static_cast<TRIPAttrTypeFlag>(type))
        {
            case TRIP_WITHDRAWN_ROUTE:
            {
                _withdrawnRoute = new TRIPWithdrawnRoute(data+cLen);
                cLen += _withdrawnRoute->getLength();
            }
            break;
            case TRIP_REACHABLE_ROUTE:
            {
                _reachableRoute = new TRIPReachableRoute(data+cLen);
                cLen += _reachableRoute->getLength();
            }
            break;
            case TRIP_NEXT_HOP_SERVER:
            {
                _nextHopServer = new TRIPNextHopServer(data+cLen);
                cLen += _nextHopServer->getLength();
            }
            break;
            case TRIP_ADVERTISEMENT_PATH:
            {
                _advertisePath = new TRIPAdvertisePath(data+cLen);
                cLen += _advertisePath->getLength();
            }
            break;
            case TRIP_ROUTED_PATH:
            {
                _routedPath = new TRIPRoutedPath(data+cLen);
                cLen += _routedPath->getLength();
            }
            break;
            case TRIP_ATOMIC_AGGREGATE:
            {
                _atomicAggregate = new TRIPAtomicAggregate(data+cLen);
                cLen += _atomicAggregate->getLength();
            }
            break;
            case TRIP_LOCAL_PREF:
            {
                _localPref = new TRIPLocalPref(data+cLen);
                cLen += _localPref->getLength();
            }
            break;
            case TRIP_MULTI_EXIT_DISC:
            {
                _multiExitDisc = new TRIPMultiExitDisc(data+cLen);
                cLen += _multiExitDisc->getLength();
            }
            break;
            case TRIP_COMMUNITY:
            {
                _community = new TRIPCommunity(data+cLen);
                cLen += _community->getLength();
            }
            break;
            case TRIP_ITAD_TOPOLOGY:
            {
                _itadTopology = new TRIPItadTopology(data+cLen);
                cLen += _itadTopology->getLength();
            }
            break;
            case TRIP_AUTHENTICATION:
            {
                _authAttr = new TRIPAuthAttr(data+cLen);
                cLen += _authAttr->getLength();
            }
            break;
            default:
                cpLog(LOG_DEBUG, "Unsupported attribute type (%d) with length (%d)", type, len);
                cLen += len;
            break;
        }
    }
    setMsgLength(cLen + 3);
}

void 
TRIPUpdateMsg::parse(const char* data) throw (TRIPBadDataException&) 
{
    TRIPHeader hdr(data);
    parse(hdr, data+3);
}


void 
TRIPUpdateMsg::writeData(int id) const throw(VNetworkException&)
{
    TRIPMsg::writeData(id);
    _withdrawnRoute->writeData(id);
    _reachableRoute->writeData(id);
    _nextHopServer->writeData(id);
    _advertisePath->writeData(id);
    _routedPath->writeData(id);
    if(_atomicAggregate != 0) _atomicAggregate->writeData(id);
    if(_localPref != 0) _localPref->writeData(id);
    if(_multiExitDisc != 0) _multiExitDisc->writeData(id);
    if(_community != 0) _community->writeData(id);
    if(_itadTopology != 0) _itadTopology->writeData(id);
    if(_authAttr != 0) _authAttr->writeData(id);
}

void 
TRIPUpdateMsg::checkSanity() const throw (TRIPBadDataException&)
{
    if(!(_hdr->isAUpdateHdr()))
    {
        cpLog(LOG_ALERT, "Bad Header");
        //TRIPErrorHandler::instance().handleUpdateError(TRIP_HDR_ERROR, 0);
        throw TRIPBadDataException("Bad update msg header",
                                   __FILE__, __LINE__, 0);
    }
    if(_withdrawnRoute.getPtr() == 0)
    {
        cpLog(LOG_ALERT, "Missing mandatory attribute (WithdrawnRoute) in Msg");
        throw TRIPBadDataException("Missing mandatory attribute (withdrawnRoute) in Msg",
                                    __FILE__, __LINE__, 0);
    }
    if(_reachableRoute.getPtr() == 0)
    {
        cpLog(LOG_ALERT, "Missing mandatory attribute (ReachableRoute) in Msg");
        throw TRIPBadDataException("Missing mandatory attribute (ReachableRoute) in Msg",
                                    __FILE__, __LINE__, 0);
    };
    if(_nextHopServer.getPtr() == 0)
    {
        cpLog(LOG_ALERT, "Missing mandatory attribute (NextHopServer) in Msg");
        throw TRIPBadDataException("Missing mandatory attribute (NextHopServer) in Msg",
                                    __FILE__, __LINE__, 0);
    };
    if(_advertisePath.getPtr() == 0)
    {
        cpLog(LOG_ALERT, "Missing mandatory attribute (AdvertisePath) in Msg");
        throw TRIPBadDataException("Missing mandatory attribute (AdvertisePath) in Msg",
                                    __FILE__, __LINE__, 0);
    };
    if(_routedPath.getPtr() == 0)
    {
        cpLog(LOG_ALERT, "Missing mandatory attribute (RoutedPath) in Msg");
        throw TRIPBadDataException("Missing mandatory attribute (RoutedPath) in Msg",
                                    __FILE__, __LINE__, 0);
    };
}


///
void 
TRIPUpdateMsg::addAttribute(Sptr<TRIPRouteAttr> attr)
{
    int len = 0;
    switch(attr->getHeader().getAttrType())
    {
        case TRIP_NEXT_HOP_SERVER:
        {
            _nextHopServer = attr;
            len = _nextHopServer->getLength();
        }
        break;
        case TRIP_ATOMIC_AGGREGATE:
        {
            _atomicAggregate = attr;
            len = _atomicAggregate->getLength();
        }
        break;
        case TRIP_LOCAL_PREF:
        {
            _localPref = attr;
            len = _localPref->getLength();
        }
        break;
        case TRIP_MULTI_EXIT_DISC:
        {
            _multiExitDisc = attr;
            len = _multiExitDisc->getLength();
        }
        break;
        case TRIP_COMMUNITY:
        {
            _community = attr;
            len = _community->getLength();
        }
        break;
        case TRIP_ITAD_TOPOLOGY:
        {
            _itadTopology = attr;
            len = _itadTopology->getLength();
        }
        break;
        case TRIP_AUTHENTICATION:
        {
            _authAttr = attr;
            len = _authAttr->getLength();
        }
        break;
        default:
        {
            cpLog(LOG_INFO, "Unsupported attribute , ignoring..");
        }
        break;
    }
    setMsgLength(getMsgLength() + len);
}


void
TRIPUpdateMsg::addReachableRoute(Sptr<TRIPReachableRoute> route)
{
    assert(_reachableRoute.getPtr() == 0);
    _reachableRoute = route;
    int len = getMsgLength();
    len = len + _reachableRoute->getLength();
    setMsgLength(len);
}
       
void
TRIPUpdateMsg::addAdvertisePath(Sptr<TRIPAdvertisePath> path)
{
    assert(_advertisePath.getPtr() == 0);
    _advertisePath = path;
    int len = getMsgLength();
    len = len + _advertisePath->getLength();
    setMsgLength(len);
}

void
TRIPUpdateMsg::addRoutedPath(Sptr<TRIPRoutedPath> path)
{
    assert(_routedPath.getPtr() == 0);
    _routedPath = path;
    int len = getMsgLength();
    len = len + _routedPath->getLength();
    setMsgLength(len);
}


void
TRIPUpdateMsg::addWithdrawnRoute(Sptr<TRIPWithdrawnRoute> route)
{
    assert(_withdrawnRoute.getPtr() == 0);
    _withdrawnRoute = route;
    int len = getMsgLength();
    len = len + _withdrawnRoute->getLength();
    setMsgLength(len);
}


