#ifndef TRIPUpdateMsg_hxx
#define TRIPUpdateMsg_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */

static const char* const TRIPUpdateMsg_hxx_Version = 
"$Id: TRIPUpdateMsg.hxx,v 1.5 2000/11/08 22:58:13 surendra Exp $";

#include <vector>

#include "TRIPMsg.hxx"
#include "TRIPId.hxx"
#include "TRIPWithdrawnRoute.hxx"
#include "TRIPReachableRoute.hxx"
#include "TRIPAdvertisePath.hxx"
#include "TRIPRoutedPath.hxx"
#include "TRIPNextHopServer.hxx"
#include "TRIPAtomicAggregate.hxx"
#include "TRIPLocalPref.hxx"
#include "TRIPCommunity.hxx"
#include "TRIPMultiExitDisc.hxx"
#include "TRIPItadTopology.hxx"
#include "TRIPAuthAttr.hxx"


///TRIPUpdateMsg

class TRIPUpdateMsg : public TRIPMsg
{
    public:
       ///
       TRIPUpdateMsg();

       ///
       bool isAUpdateMsg() const { return true; }; 

       ///
       TRIPOpCode getType() const  { return TRIP_OP_UPDATE; };

       ///
       virtual void dump(ostream& os)const;


       ///Writes object to given network socket in network byte order
       void writeData(int id) const throw(VNetworkException&);

       ///
       string className() const { return "TRIPUpdateMsg"; };

       ///Checks the sanity of TRIP Msg, and throws a Bad Data exception
       void checkSanity() const throw (TRIPBadDataException&);

       //Accessor
       ///
       const Sptr<TRIPWithdrawnRoute>& getWithdrawnRoute() const 
       { 
           return _withdrawnRoute; 
       }
       ///
       const Sptr<TRIPReachableRoute>& getReachableRoute() const 
       { 
           return _reachableRoute; 
       }
       ///
       const Sptr<TRIPRouteAttr>& getNextHopServer() const 
       { 
           return _nextHopServer; 
       }
       ///
       const Sptr<TRIPAdvertisePath>& getAdvertisePath() const 
       { 
           return _advertisePath; 
       }
       ///
       const Sptr<TRIPRoutedPath>& getRoutedPath() const 
       { 
           return _routedPath; 
       }

       ///
       bool hasAtomicAggregate() const { return (_atomicAggregate != 0); }

       ///
       const Sptr<TRIPRouteAttr>& getAtomicAggregate() const 
       { 
           return _atomicAggregate; 
       }

       ///
       bool hasLocalPref() const { return (_localPref != 0); }

       ///
       const Sptr<TRIPRouteAttr>& getLocalPref() const 
       { 
           return _localPref; 
       }

       ///
       bool hasCommunity() const { return (_community != 0); }

       ///
       const Sptr<TRIPRouteAttr>& getCommunity() const 
       { 
           return _community; 
       }

       ///
       bool hasMultiExitDisc() const { return (_multiExitDisc != 0); }

       ///
       const Sptr<TRIPRouteAttr>& getMultiExitDisc() const 
       { 
           return _multiExitDisc; 
       }

       ///
       bool hasItadTopology() const { return (_itadTopology != 0); }

       ///
       const Sptr<TRIPRouteAttr>& getItadTopology() const 
       { 
           return _itadTopology; 
       }

       ///
       bool hasAuthAttr() const { return (_authAttr != 0); }

       ///
       const Sptr<TRIPRouteAttr>& getAuthAttr() const 
       { 
           return _authAttr; 
       }

       ///Add Attribute to the message
       void addAttribute(Sptr<TRIPRouteAttr> attr);

       ///
       void addWithdrawnRoute(Sptr<TRIPWithdrawnRoute> route);

       ///
       void addReachableRoute(Sptr<TRIPReachableRoute> route);

       ///
       void addAdvertisePath(Sptr<TRIPAdvertisePath> path);

       ///
       void addRoutedPath(Sptr<TRIPRoutedPath> path);

    private:
       ///
       void parse(const TRIPHeader& hdr,
                          const char* data) throw (TRIPBadDataException&);

       ///
       void parse(const char* data) throw (TRIPBadDataException&);

       Sptr<TRIPWithdrawnRoute>    _withdrawnRoute;
       Sptr<TRIPReachableRoute>    _reachableRoute;
       Sptr<TRIPAdvertisePath>    _advertisePath;
       Sptr<TRIPRoutedPath>    _routedPath;
       Sptr<TRIPRouteAttr>    _nextHopServer;
       Sptr<TRIPRouteAttr>    _atomicAggregate;
       Sptr<TRIPRouteAttr>    _localPref;
       Sptr<TRIPRouteAttr>    _community;
       Sptr<TRIPRouteAttr>    _multiExitDisc;
       Sptr<TRIPRouteAttr>    _itadTopology;
       Sptr<TRIPRouteAttr>    _authAttr;

       ///
       friend class TRIPMsgParser;
};

#endif
