
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const TRIPUtil_cxx_Version = 
"$Id: TRIPUtil.cxx,v 1.1 2000/10/26 19:51:08 surendra Exp $";
#include <assert.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>

#include "TRIPUtil.hxx"
#include "VNetworkException.hxx"
#include "cpLog.h"
 
void 
TRIPUtil::writeData(int id, const void* data, u_int32_t len) throw (VNetworkException&)
{
   size_t nleft;
   ssize_t nwritten;
   char *ptr = (char*)data;
   nleft = len;

   while(nleft > 0)
   {
       if((nwritten = ::write(id, ptr, nleft)) <= 0)
       {
           if(errno == EINTR) nwritten = 0;  //Write again
           else
           {
                 char buf[256];
                 sprintf(buf, "Failed to write data,reason:%s", strerror(errno));
                 cpLog(LOG_ALERT, buf);
                 throw VNetworkException(buf, __FILE__, __LINE__, errno);
           }
       }
       nleft -= nwritten;
       ptr += nwritten;
   }
//   cpLog(LOG_DEBUG, "Bytes written:%d" , nwritten);
}

int
TRIPUtil::readData(int connId, void *dataRead, size_t nchar, int &bRead)
{
    size_t nleft;
    ssize_t nread;

    char *ptr = (char*)dataRead;

    nleft = nchar;

    while(nleft > 0)
    {
        if((nread  =read(connId, ptr, nleft)) < 0) {
            if(errno == EINTR) nread = 0;
            else {
              return -1;
            }
        } else if(nread == 0) {
            //NO more data
            return 0;
        }
        nleft -= nread;
        ptr += nread;
        bRead += nread;
    }
    return(1);
}
