
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */



static const char* const CommandLine_cxx_Version =
    "$Id: CommandLine.cxx,v 1.17 2000/10/02 17:55:35 veer Exp $";



#include <iostream.h>
#include "CommandLine.hxx"
#include "Daemon.hxx"
#include "cpLog.h"


///
Sptr < CommandLine > CommandLine::commandLine = 0;
///
const pair < const char*, const char* > CommandLine::cmdLineOptionString[] =
    {
        pair < const char*, const char* > ("pshostname", "ps.private.vovida.com"),
        pair < const char*, const char* > ("psremoteport", "6005" ),
        pair < const char*, const char* > ("sipport", "5060" ),
        pair < const char*, const char* > ("retransmit", "0" ),
        pair < const char*, const char* > ("daemon", "0" ),
        pair < const char*, const char* > ("debuglevel", "LOG_INFO" ),
        pair < const char*, const char* > ("logfile", "" ),
        pair < const char*, const char* > ("psbackuphostname", "ps.private.vovida.com"),
        pair < const char*, const char* > ("psbackupremoteport", "6010" ),
        pair < const char*, const char* > ("retransinitial", "500" ),
        pair < const char*, const char* > ("retransmax", "4000" ),
    };
///
Sptr < CommandLine >
CommandLine::instance(
    const int argc,
    const char** argv,
    const char* applicationUsage
)
{
    if (argc <= 1)
    {
        cerr << "Usage: " << argv[0] << " " << applicationUsage << endl;
        exit( -1);
    }

    if (commandLine == 0 )
    {
        commandLine = new CommandLine();
        assert( commandLine != 0 );
        commandLine->setDefaultValues();
        commandLine->parseCommandLine( argc, argv );

        // Run as a daemon
        if (commandLine->getInt( "daemon" ) == 0)
        {
            if (Daemon() != 0 )
            {
                cerr << "Cannot start as daemon!" << endl;
                exit( -1);
            }
        }

        // Set the debug level
        string debugLevel = CommandLine::instance()->getString( "debuglevel" );

        int priority = cpLogStrToPriority(debugLevel.c_str());
        if (priority)
        {
            cpLog( LOG_ALERT, "set level to %s", cpLogPriorityToStr(priority));
            cpLogSetPriority(priority);
        }
        else
        {
            cpLog( LOG_ALERT, "set level to LOG_DEBUG");
            cpLogSetPriority( LOG_DEBUG );
        }

        string logfile = commandLine-> getString( "logfile" );

        if (logfile != "")
        {
            cpLog( LOG_ALERT, "setting log file to %s", logfile.c_str());
            cpLogOpen(logfile.c_str());
        }

        commandLine->print(LOG_INFO);
    }
    return commandLine;
}

///
void
CommandLine::setDefaultValues()
{
    for (unsigned int i = 0;
            i < sizeof(cmdLineOptionString) / sizeof(cmdLineOptionString[0]);
            i++)
    {
        cmdLineOptionTable.insert(pair < string, string > (
                                      CommandLine::cmdLineOptionString[i].first,
                                      CommandLine::cmdLineOptionString[i].second ));
    }
}

///
void
CommandLine::parseCommandLine( const int argc, const char**argv )
{
    int option_index = 0;
    int arg_count = 0;

    int c = 0;

#if defined(__svr4__)|| defined (__SUNPRO_CC)

    struct option
    {
# if defined __STDC__ && __STDC__
        const char *name;
# else
       const char *name;
# endif 
        /* has_arg can't be an enum because some compilers complain about
           type mismatches in all the code that assumes it is an int.  */
        int has_arg;
        int *flag;
        int val;
    };

#endif

    static struct option long_options[] =
        {
            {
                CommandLine::cmdLineOptionString[0].first, 1, 0, 0
            },
            { CommandLine::cmdLineOptionString[1].first, 1, 0, 0},
            { CommandLine::cmdLineOptionString[2].first, 1, 0, 0},
            { CommandLine::cmdLineOptionString[3].first, 1, 0, 0},
            { CommandLine::cmdLineOptionString[4].first, 1, 0, 0},
            { CommandLine::cmdLineOptionString[5].first, 1, 0, 0},
            { CommandLine::cmdLineOptionString[6].first, 1, 0, 0},
            { CommandLine::cmdLineOptionString[7].first, 1, 0, 0},
            { CommandLine::cmdLineOptionString[8].first, 1, 0, 0},
            { CommandLine::cmdLineOptionString[9].first, 1, 0, 0},
            { CommandLine::cmdLineOptionString[10].first, 1, 0, 0},
        };



    while ( (c = getopt ((argc),
                         const_cast < char** > (argv),
                         "drv:f:m:R:") ) != EOF)
    {
        switch (c)
        {
            case 0:
            {
                // If this option set a flag, do nothing else now

                if (long_options[option_index].flag != 0 )
                    break;


                TableIter i =
                    cmdLineOptionTable.find(long_options[option_index].name);

                if (i != cmdLineOptionTable.end() )
                {
                    cmdLineOptionTable[long_options[option_index].name]
                    = optarg;
                }
                else
                {
                    cpLog(LOG_DEBUG,
                          "Skipping Command Line Option. Option %s Value Type %s",
                          long_options[option_index].name,
                          optarg);
                }
            }
            break;

            case 'd':
            {
                // daemon flag
                cmdLineOptionTable[long_options[4].name] = "1";
            }
            break;

            case 'r':
            {
                // retransmit flag
                cmdLineOptionTable[long_options[3].name] = "1";
            }
            break;

            case 'f':
            {
                // filename flag
                cmdLineOptionTable[long_options[6].name] = optarg;
            }
            break;

            case 'v':
            {
                // verbose (debug msg) flag
                int priority =
                    cpLogStrToPriority(optarg);

                if (priority != -1)
                {
                    cmdLineOptionTable[long_options[5].name] = optarg;
                }
                else
                {
                    printf("Invalid log level: '%s'\n", optarg);
                    exit( -1);
                }

            }
            break;

            case 'R':
            {
                // get initial retransmission time
                int initial = atoi(optarg);

                cmdLineOptionTable[long_options[9].name] = optarg;
            }
            break;

            case 'M':
            {
                // get initial retransmission time
                int max = atoi(optarg);

                cmdLineOptionTable[long_options[10].name] = optarg;
            }
            break;

            case '?':  // getopt_long already printed an error message
            break;

            default:
            printf ("?? getopt returned character code 0%o ??\n", c);

            // abort ();
        }

    }

    if ((argc - optind) < 3)
    {
        printf("Not enough arguments\n");
        exit( -1);
    }

    if ((argc - optind) > 3)
    {
        printf("Too many arguments\n");
        exit( -1);
    }

    if (optind < argc)
    {
        // printf ("non-option ARGV-elements...\n");
        while (optind < argc)
        {
            if (arg_count == 0)
            {
                // first required arg must be the host name/port.
                const string& psName = argv[optind];
                string::size_type start;
                string::size_type myLen;

                start = psName.find(":");
                myLen = psName.length();

                // printf("host:%s\n", (psName.substr(0, start)).c_str());
                // printf("port:%s\n", (psName.substr(start+1, myLen)).c_str());


                // Set the provisioning server hostname info
                cmdLineOptionTable[long_options[0].name] =
                    (psName.substr(0, start)).c_str();

                // Check to see if the port info is given...
                if (start != string::npos )
                {
                    // Overwrite default port info only if it's specified
                    cmdLineOptionTable[long_options[1].name] =
                        (psName.substr(start + 1, myLen)).c_str();
                }

            }
            else if (arg_count == 1)
            {
                // first required arg must be the host name/port.
                const string& psName = argv[optind];
                string::size_type start;
                string::size_type myLen;

                start = psName.find(":");
                myLen = psName.length();

                // printf("host:%s\n", (psName.substr(0, start)).c_str());
                // printf("port:%s\n", (psName.substr(start+1, myLen)).c_str());


                // Set the provisioning server hostname info
                cmdLineOptionTable[long_options[7].name] =
                    (psName.substr(0, start)).c_str();

                // Check to see if the port info is given...
                if (start != string::npos )
                {
                    // Overwrite default port info only if it's specified
                    cmdLineOptionTable[long_options[8].name] =
                        (psName.substr(start + 1, myLen)).c_str();
                }

            }
            else
            {
                // second required arg must be the sip port.

                cmdLineOptionTable[long_options[2].name] = argv[optind];

                // printf ("sipport:%s\n", argv[optind]);

            }


            arg_count++;
            optind++;
        }
    }
}


///
Sptr < CommandLine >
CommandLine::instance()
{
    if (commandLine == 0 )
    {
        assert( commandLine != 0 );
        return 0;
    }
    else
    {
        return commandLine;
    }
}



///
const string&
CommandLine::getString(const string& cmdOption)
{
    static string nullstring = "";

    TableIter i = cmdLineOptionTable.find(cmdOption);

    if ( i != cmdLineOptionTable.end() )
        return i->second;

    return nullstring;
}

///
const int
CommandLine::getInt(const string& cmdOption)
{
    TableIter i = cmdLineOptionTable.find(cmdOption);

    int ret = 0;

    if ( i != cmdLineOptionTable.end() )
        ret = atoi(i->second.c_str());

    return ret;
}

void
CommandLine::print(const int loglevel)
{
    cpLog(loglevel, "************************************");

    cpLog(loglevel, "pshostname: %s",
          commandLine->getString( "pshostname" ).c_str());

    cpLog(loglevel, "psremoteport: %d" , commandLine->getInt( "psremoteport" ));

    cpLog(loglevel, "psbackuphostname: %s",
          commandLine->getString( "psbackuphostname" ).c_str());

    cpLog(loglevel, "psbackupremoteport: %d" ,
          commandLine->getInt( "psbackupremoteport" ));

    cpLog(loglevel, "sipport: %d" , commandLine->getInt( "sipport" ));

    cpLog(loglevel, "retransmit: %d", commandLine->getInt( "retransmit" ));

    cpLog(loglevel, "daemon: %d" , commandLine->getInt( "daemon" ));

    cpLog(loglevel, "debuglevel: %s" ,
          commandLine->getString( "debuglevel" ).c_str());

    cpLog(loglevel, "logfile: %s" , commandLine->getString( "logfile" ).c_str());

    cpLog(loglevel, "retransinitial: %d" ,
          commandLine->getInt( "retransinitial" ));

    cpLog(loglevel, "retransmax: %d" ,
          commandLine->getInt( "retransmax" ));

    cpLog(loglevel, "compiled at: %s", __DATE__ " " __TIME__);

    cpLog(loglevel, "************************************");
}


// End of File
