
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CopyOnWriteData_cxx_Version =
    "$Id: CopyOnWriteData.cxx,v 1.10 2000/11/09 22:09:48 sunitha Exp $";


//Authors: Sunitha Kumar, Cullen Jennings

#include "CopyOnWriteData.hxx"
#include <cstdio>
#include <ctype.h>

#include "cpLog.h"

#if USE_HASH_MAP
#include <hash_map>
#endif

#define INLINE_ inline

//static const char* emptyStr = "";

static const int baseMultiple = 32;

//CopyOnWriteData::const int npos = -1;

INLINE_
CopyOnWriteData::CopyOnWriteData()
        :
        data_()
{}



INLINE_
CopyOnWriteData::CopyOnWriteData( const char* str, int length )
        :
        data_(str, length)
{}



INLINE_
CopyOnWriteData::CopyOnWriteData( const char* str )
        :
        data_(str, mystrlen(str))
{}



INLINE_
CopyOnWriteData::CopyOnWriteData( const string& str )
        :
        data_(str.c_str(), str.length())
{}



INLINE_
CopyOnWriteData::CopyOnWriteData( const mstring& mstr )
        :
        data_(mstr.c_str(), mstr.length())
{}



INLINE_
CopyOnWriteData::CopyOnWriteData( int value )
        :
        data_(8)
{
    char str[256];
    sprintf(str, "%d", value);
    int tmp = mystrlen(str);
    data_.makeWritable(tmp);
    memcpy(data_.bitsPtr, str, tmp);
    data_.size = tmp;
}


INLINE_
CopyOnWriteData::CopyOnWriteData( const CopyOnWriteData& data )
        :
        data_(data.data_)
{}



INLINE_
CopyOnWriteData&
CopyOnWriteData::operator=( const char* str )
{
    int tmp = mystrlen(str);

    data_.clear();
    data_.makeWritable(tmp);
    memcpy(data_.bitsPtr, str, tmp);
    data_.size = tmp;
    return *this;
}


INLINE_
CopyOnWriteData&
CopyOnWriteData::operator=( const CopyOnWriteData& data )
{
    if (this != &data)
    {
        data_.clear();
        data_ = data.data_;
    }
    return *this;
}


INLINE_
const char*
CopyOnWriteData::getData() const
{
    data_.makeCString();
    return data_.bitsPtr;
}


INLINE_
const char*
CopyOnWriteData::getDataBuf() const
{
    return data_.bitsPtr;
}


INLINE_
char
CopyOnWriteData::getChar(int i) const
{
    assert(i < data_.size);
    return data_.bitsPtr[i];
}


INLINE_
void
CopyOnWriteData::setchar(int i, char c)
{
    data_.makeWritable(i + 1);

    if (i >= data_.size)
    {
        // do something now
        data_.size = i + 1;
    }
    data_.bitsPtr[i] = c;
}


INLINE_
char
CopyOnWriteData::operator[](int i) const
{
    return getChar(i);
}


INLINE_
int
CopyOnWriteData::length() const
{
    return data_.size;
}


INLINE_
int
CopyOnWriteData::compare(const char* str, int length) const
{
    return data_.compare(str, length);
}


INLINE_
int
CopyOnWriteData::compare(const CopyOnWriteData& data) const
{
    return data_.compare(data.data_.bitsPtr, data.data_.size);
}

INLINE_
int
CopyOnWriteData::compareNoCase(const char* str, int length) const
{
    return data_.compareNoCase(str, length);
}


INLINE_
int
CopyOnWriteData::compareNoCase(const CopyOnWriteData& data) const
{
    return data_.compareNoCase(data.data_.bitsPtr, data.data_.size);
}


INLINE_
bool
CopyOnWriteData::operator==(const char* str) const
{
    return ( data_.compare(str, mystrlen(str)) == 0 );
}


INLINE_
bool
CopyOnWriteData::operator==( const CopyOnWriteData& data) const
{
    return ( data_.compare(data.data_.bitsPtr, data.data_.size) == 0 );
}


INLINE_
bool
CopyOnWriteData::operator!=(const char* str) const
{
    return !operator==(str);
}


INLINE_
bool
CopyOnWriteData::operator!=(const CopyOnWriteData& data) const
{
    return !operator==(data);
}


INLINE_
bool
CopyOnWriteData::operator>(const CopyOnWriteData& data) const
{
    return ( data_.compare(data.data_.bitsPtr, data.data_.size) > 0 );
}


INLINE_
bool
CopyOnWriteData::operator<(const CopyOnWriteData& data) const
{
    return ( data_.compare(data.data_.bitsPtr, data.data_.size) < 0 );
}


INLINE_
CopyOnWriteData
CopyOnWriteData::operator+(const CopyOnWriteData& data) const
{
    CopyOnWriteData tmp(*this);
    tmp += data;
    return tmp;
}


INLINE_
CopyOnWriteData
CopyOnWriteData::operator+(const char* str) const
{
    CopyOnWriteData tmp(*this);
    tmp += str;
    return tmp;
}


INLINE_
void
CopyOnWriteData::operator+=(const CopyOnWriteData& data)
{
    if (data.length())
    {
        data_.makeWritable(length() + data.length());
        memcpy(data_.bitsPtr + data_.size,
               data.data_.bitsPtr, data.length());
        data_.size += data.length();
    }
}


INLINE_
void
CopyOnWriteData::operator+=(const char* str)
{
    int len = mystrlen(str);
    if (len)
    {
        data_.makeWritable(length() + len);
        memcpy(data_.bitsPtr + data_.size, str, len);
        data_.size += len;
    }
}


INLINE_
CopyOnWriteData::~CopyOnWriteData()
{}



INLINE_
void
CopyOnWriteData::erase()
{
    data_.clear();
}


INLINE_
CopyOnWriteData::operator string() const
{
    return string(data_.bitsPtr, static_cast < unsigned int > (length()));
}


INLINE_
CopyOnWriteData::operator const char*() const
{
    return getData();
}


INLINE_
CopyOnWriteData::operator mstring() const
{
    return mstring(getData());
}


INLINE_
CopyOnWriteData::operator int() const
{
    int l = data_.size;
    int val = 0;
    char* p = data_.bitsPtr;

    while (l--)
    {
        char c = *p++;
        if ((c >= '0') && (c <= '9'))
        {
            val *= 10;
            val += c - '0';
        }
        else
        {
            return val;
        }
    }
    return val;
    //    return atoi(getData()); /// can we make this faster ???
}



INLINE_
string CopyOnWriteData::getstring() const
{
    return string(data_.bitsPtr, static_cast < unsigned int > (length()));
}


#if USE_HASH_MAP
INLINE_
size_t
CopyOnWriteData::hashfn() const
{
    hash < const char* > x;
    return x(getData());
#if 0
    hash_val = x(getData());
    if (!hash_valid)
    {
        hash_valid = true;
    }
    return hash_val;
#endif
}
#endif


//#if 0
INLINE_
int
CopyOnWriteData::find( const CopyOnWriteData& match, int start )
{
    int pos = 0;
    if ( start != npos )
    {
        pos = start;
    }

    const char* ptr = data_.bitsPtr + pos;
    while ((pos + match.data_.size) <= data_.size)
    {
        if ( (*(ptr) == *match.data_.bitsPtr ) &&
                (strncmp(ptr,
                         match.data_.bitsPtr,
                         match.data_.size)
                 == 0))
        {
            // a match has been found
            return pos;
        }
        ++ptr;
        ++pos;  // or not
    }

    return npos;
}
//#endif

INLINE_
int
CopyOnWriteData::find( const char* match, int start )
{
    int matchLength = mystrlen(match);

    int pos = 0;
    if ( start != npos )
    {
        pos = start;
    }

    int compares = data_.size - matchLength - pos;
    const char* ptr = data_.bitsPtr + pos;
    while ((pos + matchLength) <= data_.size)
        //    while((pos + matchLength) <= data_.size)
        while (compares-- >= 0)
        {
            if (
                (*match == *(ptr)) &&
                ( strncmp(ptr,
                          match,
                          matchLength)
                  == 0))
            {
                // a match has been found
                return pos;
            }
            ++ptr;
            ++pos;  // or not
        }

    return npos;
}


INLINE_
int
CopyOnWriteData::match( const char* match,
                        CopyOnWriteData* retModifiedCopyOnWriteData,
                        bool doReplace,
                        CopyOnWriteData replaceWith)
{
    if (replaceWith != "")
    {
        int retVal;
        string::size_type pos = getstring().find(match);

        if (pos == string::npos)
        {
            return NOT_FOUND;
        }

        int matchLength = mystrlen(match);

        string::size_type replacePos = pos + matchLength;
        retVal = FIRST;

        if (retModifiedCopyOnWriteData)
        {
            (*retModifiedCopyOnWriteData) = getstring().substr(0, pos);
            if (retModifiedCopyOnWriteData->length())
            {
                retVal = FOUND;
            }
        }

        if (doReplace)
        {
            if (replacePos <= static_cast < unsigned int > (length()) )
            {
                replace(0, replacePos, replaceWith);
            }
            else
            {
                //	    printf("buf =<%s> match =<%s>\n", buf , match.getData() );
                printf("pos=%d  match.len=%d replacePos=%d  buf.size()= %d\n",
                       pos,
                       matchLength,
                       replacePos,
                       length() );

            }
        }

        return retVal;
    }
    else
    {
        int retVal;
        int pos = find(match);

        if (pos == npos)
        {
            return NOT_FOUND;
        }

        int matchLength = mystrlen(match);

        int replacePos = pos + matchLength;
        retVal = FIRST;

        if (retModifiedCopyOnWriteData)
        {
            (*retModifiedCopyOnWriteData) = *this;
            retModifiedCopyOnWriteData->data_.substr(0, pos);
            if (retModifiedCopyOnWriteData->length())
            {
                retVal = FOUND;
            }
        }

        if (doReplace)
        {
            if (replacePos <= length() )
            {
                data_.truncate(replacePos, 0);
            }
            else
            {
                //	    printf("buf =<%s> match =<%s>\n", buf , match.getData() );
                printf("pos=%d  match.len=%d replacePos=%d  buf.size()= %d\n",
                       pos,
                       matchLength,
                       replacePos,
                       length() );
            }
        }

        return retVal;
    }
}


INLINE_
bool isEqualNoCase(const CopyOnWriteData& left, const CopyOnWriteData& right)
{
    return (left.compareNoCase(right) == 0);
}

INLINE_
bool isEqualNoCase(const char* left, const CopyOnWriteData& right)
{
    return (right.compareNoCase(left, mystrlen(left)) == 0);
}



INLINE_
void
CopyOnWriteData::replace(int startpos, int endpos,
                         const CopyOnWriteData& replaceStr)
{
    if (length())
    {
        string str = getstring();

        str.replace(startpos, endpos, replaceStr.getstring());
        *this = CopyOnWriteData(str);
    }
}


INLINE_
void CopyOnWriteData::removeSpaces()
{
   if (length() == 0)
    {
        return ;
    }
 
    int firstChar = 0;
 
    while ((firstChar < length()) && (getChar(firstChar) == ' '))
    {
        firstChar++;
    }
 
    int lastChar = length();
    while ((lastChar > 0) && (getChar(lastChar - 1) == ' '))
    {
        lastChar--;
    }                      
    data_.substr(firstChar, lastChar);       
}


INLINE_
void CopyOnWriteData::removeLWS()
{
    int pos = find ("\r\n");
    bool replaceFlag = false;
    do
    {
        int replaceAfter = pos+1;
    
      
        while ( (replaceAfter+1 < length()) && 
                ( (getChar(replaceAfter+1) == '\t') ||
                  (getChar(replaceAfter+1) == ' ')
                ) 
              )
        {
            replaceAfter++;
            replaceFlag = true;
        }
        if (replaceFlag)
        {
            int replaceBefore = pos;
            while ( (replaceBefore-1 > 0) &&
                    ( (getChar(replaceBefore-1) == '\t') ||
                      (getChar(replaceBefore-1) == ' ')
                    )
                  )
            {
                replaceBefore--;
            }
            int replaceNum = replaceAfter-replaceBefore+1;
            replace(pos, replaceNum, ""); //replace with nothing
	   
        }
        pos = find("\r\n", pos+2);
    }while (pos < length());
}

INLINE_
void
CopyOnWriteData::expand(CopyOnWriteData startFrom,
                        CopyOnWriteData findstr,
                        CopyOnWriteData replstr,
                        CopyOnWriteData delimiter)
{
    //    printf("expand is slow, so don't use it!\n");
    //    hash_valid = false;

    int startPos = find(startFrom);
    if (startPos < npos)
    {
        int delimPos
        = find(delimiter, startPos);
        int findPos
        = find(findstr, startPos);

        while (findPos < delimPos)
        {
            //found replstr, replace
            replace( findPos, findstr.length(), replstr.getstring());
            //find next.
            //delimPos = buf.find( delimiter.getData(), findPos);
            delimPos
            = find( delimiter,
                    findPos + replstr.length());
            findPos
            = find( findstr, findPos);
        }
    }
}


INLINE_
void
CopyOnWriteData::deepCopy (const CopyOnWriteData &src,
                           char ** bufPtr,
                           int *bufLenPtr)
{
    *this = src;  // no deep copy needed ?
}
