#ifndef COPY_ON_WRITE_DATA_H_
#define COPY_ON_WRITE_DATA_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CopyOnWriteDataHeaderVersion =
    "$Id: CopyOnWriteData.hxx,v 1.8 2000/11/09 22:09:49 sunitha Exp $";



//Authors: Sunitha Kumar, Cullen Jennings

#include <cstring>
#include <string>
#include "mstring.hxx"
#include "VException.hxx"
#include "CWBuffer.hxx"
#include "DataException.hxx"
#include "limits.h"

/** This will eventually implemented the copy on write, reference
    counted smart buffer scheme
*/
class CopyOnWriteData
{
    public:
        static const int npos = INT_MAX;

        // creation members.
        ///
        //Data( const string& str);
        ///
        CopyOnWriteData( );
        ///
        CopyOnWriteData( const char* str );
        ///
        CopyOnWriteData( const char* buffer, int length );
        ///
        CopyOnWriteData( const CopyOnWriteData& data );
        ///
        CopyOnWriteData( const string& str);
        ///
        CopyOnWriteData( const mstring& mstr);
        ///
        CopyOnWriteData( const int value);

        ///
        ~CopyOnWriteData();

        ///
        bool operator>(const CopyOnWriteData& ) const ;
        ///
        bool operator<(const CopyOnWriteData& ) const;

        ///
        CopyOnWriteData& operator=(const char* str);
        ///
        CopyOnWriteData& operator=(const CopyOnWriteData& data);
        ///
        //Data& operator=(const string& str);

        //access members
        ///
        //string getData() const;


        /// getData returns a NUL terminated (e.g. a C string) buffer
        const char* getData() const;

        /** getDataBuf differs from getData in that the resultant buffer is NOT
        NUL terminated */
        const char* getDataBuf() const;

        //void substr( int start, int length , char* str) const;
        //char* substr( int start, int length ) const;
        ///
        char getChar( int i ) const;  //return the i'th char of string.
        ///
        void setchar( int i, char c );  //write to the i'th char of string.
        ///
        char operator[]( int i ) const;  //return the i'th char of string.
        ///
        int length() const;
        ///
        bool operator==( const char* str ) const;
        ///
        bool operator==( const CopyOnWriteData& data ) const;
        ///
        //bool operator==(const string& second );
        ///
        bool operator!=( const char* str ) const;
        ///
        bool operator!=( const CopyOnWriteData& data ) const;
        ///
        int compare(const char* str, int length) const;
        ///
        int compare(const CopyOnWriteData& data) const;
        ///
        int compareNoCase(const char* str, int length) const;
        ///
        int compareNoCase(const CopyOnWriteData& data) const;
        ///
        //bool operator!=(const string& str);
        ///
        CopyOnWriteData operator+( const CopyOnWriteData& data) const;
        ///
        CopyOnWriteData operator+( const char* str) const;
        ///
        //Data& operator+(const Data& firstsrc, const string& secondsrc);

        //modification members
        ///
        void operator+=(const CopyOnWriteData&);
        ///
        void operator+=(const char*);

        ///
        void erase();
        ///
        //Data& operator+=(const string&);

        ///
        //size_t hashfn() ;

#if USE_HASH_MAP
        ///
        size_t hashfn() const;
#endif

        // conversion operators
        ///
        operator string() const;

        ///
        operator const char*() const;

        ///
        operator mstring() const;

        ///
        operator int() const;
        //destruction

        int CopyOnWriteData::match(const char* match,
                                   CopyOnWriteData* data,
                                   bool replace = false,
                                   CopyOnWriteData replaceWith = "");

        /*
          ///
          //returns the starting pos of match in this data item.
          // if replace is true, then, the matched string is replaced with the 
          //replaceWith string.
          //if match string is not found, then MATCH_NOT_FOUND exception is thrown.
          int  match( const Data& match, 
          Data* retModifiedData, 
          bool replace=false, 
          const Data& replaceWith="");
        */

        /// removes spaces before and after a string.
        void removeSpaces();
        ///
        void removeLWS();

        ///expand requird for expandin headers
        void expand(CopyOnWriteData startFrom,
                    CopyOnWriteData findstr,
                    CopyOnWriteData replstr,
                    CopyOnWriteData delimiter);

        /// do a case-insensitive match
        friend bool isEqualNoCase( const CopyOnWriteData& left,
                                   const CopyOnWriteData& right ) ;


        /// do a case-insensitive match
        friend bool isEqualNoCase( const char* left,
                                   const CopyOnWriteData& right ) ;


        ///
        void deepCopy (const CopyOnWriteData &src,
                       char ** bufPtr = 0,
                       int *bufLenPtr = 0);
        ///
        int find( const CopyOnWriteData& match, int start = npos );
        ///
        int find( const char* match, int start = npos );

    private:
        void getWritable(int length = 0);
        void truncate(int first, int last);
        void replace(int startpos, int endpos, const CopyOnWriteData& replaceStr);

        ///
        string getstring() const;


    private:
        mutable CWBuffer data_;
};

#include "CopyOnWriteData.cxx"

#endif
