#ifndef COUNT_SEMAPHORE_HXX_
#define COUNT_SEMAPHORE_HXX_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const CountSemaphoreVersion =
    "$Id: CountSemaphore.hxx,v 1.8 2000/10/02 17:55:36 veer Exp $";




#include "VMutex.h"
#include "global.h"
#include <assert.h>
#if defined(__svr4__) || defined (__SUNPRO_CC)
#define USE_REAL_LOCKING 1
#else
#define USE_REAL_LOCKING 0
#endif

#if USE_REAL_LOCKING
#define LOCK() mutex.lock()
#define UNLOCK() mutex.unlock()
#define INC(x) x++
#define DEC(x) x--
#define DEC_AND_CMP(x,r) x--,r = (x == 0)
#else

#define LOCK()
#define UNLOCK()

#define INC(x) \
({   \
asm volatile ("lock; incl %0" : : "m" (x) ); \
})

#define DEC(x) \
({   \
asm volatile ("lock; decl %0" : : "m" (x) ); \
})

#define DEC_AND_CMP(x,r) \
({   \
asm volatile ("lock; decl %1; sete %0 " : "=m" (r) : "m" (x) ); \
})


#endif

#define INLINE_ inline

///
class CountSemaphore
{
    public:
        INLINE_ CountSemaphore()
        :
#if USE_REAL_LOCKING
        mutex(),
#endif
        count(0)
        {}


        INLINE_ bool compare(int value)
        {
            bool retVal;
            //LOCK(); // why lock here ? CJ
            retVal = (value == count);
            //UNLOCK();
            return retVal;
        }

        INLINE_ void increment()
        {
            LOCK();
            INC(count);
            UNLOCK();
        }

        INLINE_ bool decrement()
        {
            bool retVal;
            assert(count > 0);
            LOCK();
            DEC_AND_CMP(count, retVal);
            UNLOCK();
            return retVal;
        }

    private:
        CountSemaphore(const CountSemaphore&);
        const CountSemaphore& operator=(const CountSemaphore&);
        bool operator==(const CountSemaphore&);

#if USE_REAL_LOCKING
        ///
        VMutex mutex;
#endif

        ///
        volatile int count;
};

#endif
