#ifndef NETWORKADDRESS_H
#define NETWORKADDRESS_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const NetworkAddress_h_fileVersion =
    "$Id: NetworkAddress.h,v 1.27 2000/09/21 22:27:08 wenqing Exp $";



#include "global.h"
#include "VMutex.h"

#include <iostream.h>
#include <string>
#include "vtypes.h"
#include <netinet/in.h>

#define IPV4_LENGTH 4

class NetworkAddress
{
    public:
        // The host is not defined yet.
        NetworkAddress ( int port = -1 );

        // The host hostName can in the same form at passed to setHostName
        NetworkAddress ( string hostName, int port = -1 );

        // takes IP or DNS with option ":port" syntax
        // it can be "lab1.vovida.com" or "192.168.4.5" or
        // "lab1.vovoda.com:80" or ...
        void setHostName ( const string& theAddress );

        // set port
        void setPort ( int port );

        // get host name, like "lab1.vovida.com"
        string getHostName () const;

        // get "192.168.4.5" kind of string
        string getIpName () const;

        // return a uint
        u_int32_t getIp4Address () const;

        // return sockaddr kind of address
        void getSockAddr(struct sockaddr & socka) const;
        //
        int getPort () const;

        /*
            /// fill buffer with IP address
            void addressToChar(char * pcAddress) const;
            /// get n'th byte of IP address
            unsigned char operator [] ( int nth) const;
        */

        /// compare two IP addresses
        friend bool operator < (const NetworkAddress & xAddress,
                                const NetworkAddress & yAddress );

        /// compare two IP addresses
        friend bool operator == (const NetworkAddress & xAddress,
                                 const NetworkAddress & yAddress );

        /// convert to string - returns same as getHostName
        operator string() ;

        friend ostream & operator<< ( ostream & xStream,
                                      const NetworkAddress& rxObj );

        /// assignment operator =
        NetworkAddress& operator=( const NetworkAddress& x );

        ostream & print ( ostream & xStr ) const;

        /// hash function, uses last 2 bytes of ip address + 2 bytes of port
        /// the bits 15-0 of ipaddress (Bigendian) becomes the Most Significant Byte(MSB) of hashkey,
        /// bits 15-0 of port becomes LSB of hashkey.
        u_int32_t hashIpPort( );

        /// hash function, uses last 2 bytes of ip address + 2 bytes of port
        static u_int32_t hashIpPort( const string& theAddress, const string& port );

        /// hash function, uses last 2 bytes of ip address + 2 bytes of port
        static u_int32_t hashIpPort( const u_int32_t ipAddress, const int port );

        ///Mutexes the gethostbyname(), return 0 when successfull
        static int getHostByName(const char* hostName,
                                 struct hostent* hEnt,
                                 char* buf,
                                 int buflen,
                                 int* thrErrno);

        static const int getHostLookupFailed = 1;
        static const int getHostLookupOK = 0;

    private:
        /// IP address value
        char ipAddress[IPV4_LENGTH];  // it's in network byte order

        /// port value
        int aPort;

        static VMutex _mutex;
};


inline ostream &
operator<< ( ostream & xStream, const NetworkAddress& rxObj )
{
    return rxObj.print ( xStream );
}


// NETWORKADDRESS_H
#endif
