#ifndef NEWDATA_HXX_
#define NEWDATA_HXX_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const NewData_hxx_Version =
    "$Id: NewData.hxx,v 1.9 2000/10/03 18:04:03 veer Exp $";



//Authors: Sunitha Kumar, Cullen Jennings

#if defined (__SUNPRO_CC)
#define USE_HASH_MAP 0
#else
#define USE_HASH_MAP 1
#endif
#include <cstring>
#include <string>
#include "mstring.hxx"
#include "VException.hxx"
#include "DataException.hxx"

/** This will eventually implemented the copy on write, reference
    counted smart buffer scheme
*/

class NewData
{

    public:
        // creation members.
        ///
        //Data( const string& str);
        ///
        NewData( );
        ///
        NewData( const char* str );
        ///
        NewData( const char* buffer, int length );
        ///
        NewData( const NewData& data );
        ///
        NewData( const string& str);
        ///
        NewData( const mstring& mstr);
        ///
        NewData( const int value);

        ///
        ~NewData();

        ///
        bool operator>(const NewData& ) const ;
        ///
        bool operator<(const NewData& ) const;

        ///
        NewData& operator=(const char* str);
        ///
        NewData& operator=(const NewData& data);
        ///
        //Data& operator=(const string& str);

        /// getData returns a NUL terminated (e.g. a C string) buffer
        const char* getData() const;

        /** getDataBuf differs from getData in that the resultant buffer is NOT
        NUL terminated */
        const char* getDataBuf() const;

        ///
        //void substr( int start, int length , char* str) const;
        //char* substr( int start, int length ) const;
        ///
        char getChar( int i ) const;  //return the i'th char of string.
        ///
        void setchar( int i, char c );  //write to the i'th char of string.
        ///
        char operator[]( int i );  //return the i'th char of string.
        ///
        int length() const;
        ///
        bool operator==( const char* str ) const;
        ///
        bool operator==( const NewData& data ) const;
        ///
        //bool operator==(const string& second );
        ///
        bool operator!=( const char* str ) const;
        ///
        bool operator!=( const NewData& data ) const;
        ///
        //bool operator!=(const string& str);
        ///
        NewData operator+( const NewData& data) const;
        ///
        NewData operator+( const char* str) const;
        ///
        //NewData& operator+(const NewData& firstsrc, const string& secondsrc);

        //modification members
        ///
        void operator+=(const NewData&);
        ///
        void operator+=(const char*);
        ///
        void replace(int startpos, int endpos, const char* replaceStr);

        ///
        void erase();
        ///
        //NewData& operator+=(const string&);

        ///
#if USE_HASH_MAP
        size_t hashfn() ;
#endif
        // conversion operators
        ///
        operator string() const;

        ///
        operator const char*() const;

        ///
        operator mstring() const;

        ///
        operator int() const;
        //destruction

        /// match
        int NewData::match(NewData match,
                           NewData* data,
                           bool replace = false,
                           NewData replaceWith = "");

        /*
            ///
            //returns the starting pos of match in this data item.
            // if replace is true, then, the matched string is replaced with the 
            //replaceWith string.
            //if match string is not found, then MATCH_NOT_FOUND exception is thrown.
            int  match( const NewData& match, 
                        NewData* retModifiedNewData, 
                        bool replace=false, 
                        const NewData& replaceWith="");
        */

        /// removes spaces before and after a string.
        void removeSpaces();

        ///expand requird for expandin headers
        void expand(NewData startFrom, NewData findstr, NewData replstr, NewData delimiter);

        /// do a case-insensitive match



        friend bool isEqualNoCase( const NewData& left, const NewData& right ) ;

        void deepCopy (const NewData &src, char ** bufPtr = 0, int *bufLenPtr = 0);

    private:
        char* buf;
        int buflen;
  //#if USE_HASH_MAP
        size_t hash_val;
  //#endif
        bool hash_valid;
        int boundary( int len ) const;
};

#endif
