#ifndef SIGNAL_ACTION_DOT_H
#define SIGNAL_ACTION_DOT_H

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */


static const char* const SignalActionHeaderVersion =
    "$Id: SignalAction.hxx,v 1.7 2000/10/26 02:41:34 icahoon Exp $";


#include "SignalSet.hxx"
#include "Writer.hxx"


/** Infrastructure common to VOCAL.
 */
namespace Vocal 
{


/** Infrastructure common to VOCAL to create and manage processes.
 */
namespace Process
{


/** An interface class used to respond to a given signal.<br><br>
 *
 *  @see    Vocal::Process::SignalHandler
 *  @see    Vocal::Process::SignalSet
 */
class SignalAction : public Vocal::IO::Writer
{
    public:


	/** Construct the action that will respond to the signals in the
	 *  given SignalSet.
	 */
        SignalAction(const SignalSet &);


	/** Virtual destructor.
	 */
        virtual ~SignalAction();


	/** Get the set of signals to be responded to.
	 */
        const SignalSet & signalSet() const;


	/** Abstract function that will be called by the 
	 *  Vocal::Process::SignalHandler when a signal in the SignalSet is 
	 *  thrown, and the action is added to the SignalHandler.
	 */
        virtual void	onSignal(int signum, int error, int code) = 0;


	/** The name of the class, "SignalAction".
	 */
        virtual const char* const name() const;


	/** Write the SignalAction to an ostream.
	 */
        virtual ostream & writeTo(ostream &) const;

    private:

        SignalSet signalSet_;

	/** Copying suppressed.
	 */
        SignalAction(const SignalAction &);


	/** Copying suppressed.
	 */
        SignalAction & operator=(const SignalAction &);
};


} // namespace Process
} // namespace Vocal


#endif // !defined(SIGNAL_ACTION_DOT_H)
