#ifndef VCONDITION_H_
#define VCONDITION_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const VConditionHeaderVersion =
    "$Id: VCondition.h,v 1.12 2000/09/20 21:38:57 cvs Exp $";




#include "VThread.h"
#include "VMutex.h"

/** Class VCondition
 ** Defines a simple conditional variable class which
 ** allows threads to block until shared data changes state.
 ** A VCondition must always be associated with a
 ** VMutex to avoid the race condition where a thread prepares to
 ** wait on a condition variable and another thread signals the
 ** condition just before the first thread actually waits on it.
 */
class VCondition
{
    public:

        /** Create an VCondition object initialized with operating system
         ** dependant defaults (if any). 
         */
        VCondition (void);

        /**
         ** Delete a VCondition object
         */
        ~VCondition (void);

        /**
         ** Block on the condition. Will return after the relativeTime, 
         ** specified in millseconds, has passed. If set to -1, the default,
         ** it will wait indefinately. Uses the specified mutex to synchronize 
         ** access to the condition. Returns 0, if successful, or an errorcode.
         */
        int wait (VMutex* mutex, int relativeTimeInMs = -1);

        /**
         ** Signal one waiting thread.
         ** Returns 0, if successful, or an errorcode.
         */
        int signal (void);

        /**
         ** Returns the operating system dependent unique id of the condition.
         */
        const vcondition_t*
        getId (void);

    private:
        /// prevent assignment and initialization
        void operator= (const VCondition &);
        VCondition (const VCondition &);

        mutable vcondition_t myId;
};

// V_CONDITION_H_
#endif
