#ifndef VMUTEX_H_
#define VMUTEX_H_

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const VMutexHeaderVersion =
    "$Id: VMutex.h,v 1.10 2000/09/20 21:38:57 cvs Exp $";




#include "VThread.h"

/**
 ** Class VMutex
 ** Defines a simple mutex class which provides
 ** 'fast' mutex functionality, meaning that if the calling thread
 ** already has the mutex locked, a call to lock will block the
 ** thread forever.
 */
class VMutex
{
    public:

        /** Create a VMutex object initialized with operating system
         ** dependent defaults (if any).
         */
        VMutex (void);

        /**
         ** Delete a VMutex object
         */
        ~VMutex (void);

        /**
         ** Lock the mutex. If the mutex is currently unlocked, it
         ** becomes locked and owned by the calling thread, and
         ** lock returns immediately. If the mutex is
         ** already locked by another thread, lock
         ** suspends the calling thread until the mutex is unlocked.
         ** If the mutex is already locked by the calling thread, the
         ** calling thread blocks forever.
         */
        void
        lock (void);

        /**
         ** Unlock the mutex.  The mutex is assumed to be locked and owned
         ** by the calling thread, meaning that a call to unlock always
         ** returns it to the unlocked state. 
         */
        void
        unlock (void);

        /**
         ** Returns the operating system dependent unique id of the mutex.
         */
        vmutex_t*
        getId (void);

    private:
        /// prevent assignment and initialization
        void operator= (const VMutex &);
        VMutex (const VMutex &);

        mutable vmutex_t myId;
};

// V_MUTEX_H_
#endif
