
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const VRWLock_cxx_Version =
    "$Id: VRWLock.cxx,v 1.5 2000/09/20 08:01:35 cvs Exp $";
#include "VRWLock.hxx"

VRWLock::VRWLock(void)
{
    writer_count = 0;
    reader_count = 0;
}

void
VRWLock::ReadLock(void)
{
    VMutex::lock();

    // wait until there are no more writers holding the lock
    while (writer_count > 0)
    {
        VMutex::unlock();
        read_barrier.Wait();
        VMutex::lock();
    }
    reader_count++;
    VMutex::unlock();
}

void
VRWLock::WriteLock(void)
{
    VMutex::lock();
    writer_count++;        // this stops new readers from getting a lock
    VMutex::unlock();
    write_lock.Wait();     // wait until the write lock is available
    VMutex::lock();
    read_barrier.Close();  // give new readers something to wait for
    VMutex::unlock();
}

void
VRWLock::Unlock(void)
{
    VMutex::lock();

    if (writer_count > 0)
    {  // we must be a writer
        writer_count--;
        if (writer_count == 0)
        {
            read_barrier.Open();  // open the floodgates
        }

        write_lock.Post();   // let it go

        VMutex::unlock();
    }
    else
    {
        reader_count--;

        // if we're the last reader and a writer is waiting, let it go
        if ((reader_count == 0) && (writer_count > 0))
        {
            write_lock.Post();
        }
        VMutex::unlock();
    }
}
