#ifndef VThreadPool_hxx
#define VThreadPool_hxx

/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const VThreadPoolHeaderVersion =
    "$Id: VThreadPool.hxx,v 1.6 2000/09/20 08:23:36 cvs Exp $";


#include <pthread.h>
#include <deque>
#include <list>

#include "VThread.h"
#include "VFunctor.hxx"
#include "VThreadInitException.hxx"
#include "Sptr.hxx"

/** Most of the code taken from "Pthreads Programming from O'REILLY publication" */

class VThreadPool
{
    public:
        typedef list < Sptr < VThread > > ListOfThreads;
        ///
        VThreadPool(int numThreads = 1);

        virtual ~VThreadPool();

        ///
        void addFunctor(const VFunctor& functor) throw(VThreadInitException&);

        ///if finsh flag is true , will wait till the entire work queue is done
        void shutdown(bool finish = true);

        const ListOfThreads& getWorkerThreads() const
        {
            return _workerThreads;
        };

    private:
        ///
        void init() throw (VThreadInitException&);

        ///
        static void* workerProc(void* arg);

        deque < VFunctor > _queue;
        pthread_mutex_t _queueLock;
        pthread_cond_t _queueNotEmpty;
        pthread_cond_t _queueNotFull;
        pthread_cond_t _queueEmpty;
        int _numThreads;
        int _queueClosed;
        int _shutdown;
        int _currQueueSize;

        ListOfThreads _workerThreads;
};

#endif
