
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */





static const char* const cpLog_cxx_Version =
    "$Id: cpLog.cxx,v 1.27 2000/10/04 22:37:58 veer Exp $";


#include <sys/types.h>
#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <errno.h>
#include <map>
#include "VThread.h"

#include "global.h"

#include "cpLog.h"
#include <string>
#include <map>

#include "VMutex.h"
#include "LockHelper.hxx"
//#ifdef __cplusplus
//extern "C" {
//#endif


// the trailing 0 is a nil pointer
static const char* priName[] =
    {
        "EMERG",
        "ALERT",
        "CRIT",
        "ERR",
        "WARNING",
        "NOTICE",
        "INFO",
        "DEBUG",
        "DEBUG_STACK",
        "DEBUG_OPER",
        "DEBUG_HB",
        0
    };


// the trailing 0 is a nil pointer
static const char* priNameShort[] =
    {
        "EMRG",
        "ALRT",
        "CRIT",
        "ERR ",
        "WARN",
        "NOTC",
        "INFO",
        "DBUG",
        "DSTK",
        "DOP ",
        "DHB ",
        0
    };

/* tbd: group the following variables into a struct */
//#define LOG_LABEL_MAX_SIZE 31
//static char cpLogLabel[LOG_LABEL_MAX_SIZE+1] = "cpLog";

//static int logPriority = LOG_ERR;
//static int logPriority = LOG_DEBUG;

#define LOG_FNAME_MAX_SIZE 256
static char cpLogFile[LOG_FNAME_MAX_SIZE + 1] = "";

static FILE* cpLogFd = stderr;

class CpLogPriority
{
    public:
        static int getPriority();
        static void setPriority(int pri);
        static void setPriorityThread(vthread_t threadId, int pri);
        static void clearPriorityThread(vthread_t threadId);

        static const char* getLabel();
        static void setLabel(const char* label);
        static void setLabelThread(vthread_t threadId, const char* label);
        static void clearLabelThread(vthread_t threadId);

    protected:
        CpLogPriority();

    private:
        static VMutex logMutex;
        static CpLogPriority* getInstance();
        static CpLogPriority* instance_;

        int logPriority;
        map < vthread_t, int > logPriorityMap;
        map < vthread_t, string > logLabelMap;
        string logLabel;
};


CpLogPriority* CpLogPriority::instance_ = 0;
VMutex CpLogPriority::logMutex;

CpLogPriority::CpLogPriority()
        : logPriority(LOG_ERR),
        logLabel("")
{}



CpLogPriority* CpLogPriority::getInstance()
{
    if (!instance_)
    {
        instance_ = new CpLogPriority;
    }
    return instance_;
}


int CpLogPriority::getPriority()
{
    map < vthread_t, int > ::iterator i;

    LockHelper lock(logMutex);
    i = getInstance()->logPriorityMap.find(VThread::selfId());

    if (i != getInstance()->logPriorityMap.end())
    {
        // found it!
        return i->second;
    }
    else
    {
        // not found, use default
        return getInstance()->logPriority;
    }
}


void
CpLogPriority::setPriority(int pri)
{
    LockHelper lock(logMutex);
    getInstance()->logPriority = pri;
}


void
CpLogPriority::setPriorityThread(vthread_t threadId, int pri)
{
    LockHelper lock(logMutex);
    getInstance()->logPriorityMap[threadId] = pri;
}


void
CpLogPriority::clearPriorityThread(vthread_t threadId)
{
    map < vthread_t, int > ::iterator i;

    LockHelper lock(logMutex);
    i = getInstance()->logPriorityMap.find(threadId);

    if (i != getInstance()->logPriorityMap.end())
    {
        getInstance()->logPriorityMap.erase(i);
    }
}


const char* CpLogPriority::getLabel()
{
    map < vthread_t, string > ::iterator i;

    LockHelper lock(logMutex);
    i = getInstance()->logLabelMap.find(VThread::selfId());

    if (i != getInstance()->logLabelMap.end())
    {
        // found it!
        return i->second.c_str();
    }
    else
    {
        // not found, use default
        return getInstance()->logLabel.c_str();
    }
}


void
CpLogPriority::setLabel(const char* label)
{
    LockHelper lock(logMutex);
    getInstance()->logLabel = label;
}


void
CpLogPriority::setLabelThread(vthread_t threadId, const char* label)
{
    LockHelper lock(logMutex);
    getInstance()->logLabelMap[threadId] = label;
}


void
CpLogPriority::clearLabelThread(vthread_t threadId)
{
    map < vthread_t, string > ::iterator i;

    LockHelper lock(logMutex);
    i = getInstance()->logLabelMap.find(threadId);

    if (i != getInstance()->logLabelMap.end())
    {
        getInstance()->logLabelMap.erase(i);
    }
}


bool
cpLogOpen (const char* f)
{
    FILE* lfd;

    if ((lfd = fopen (f, "a+")) == (FILE*)0)
    {
        cpLog (LOG_ERR, "cpLog: Cannot open %s - %s",
               f, strerror(errno));
        return false;
    }
    cpLog (LOG_INFO, "Open new log file - %s", f);
    strncpy (cpLogFile, f, LOG_FNAME_MAX_SIZE);
    cpLogFile[LOG_FNAME_MAX_SIZE] = 0;
    cpLogFd = lfd;
    return true;
}    /* cpLogOpen */

#ifndef __GNUC__
void
cpLog(int pri, const char *fmt, ...)
{
    if (pri < 0)
        return ;

    va_list ap;

    if (pri <= CpLogPriority::getPriority())
    {
        va_start(ap, fmt);
        vCpLog(pri, 0, 0, fmt, ap);
        va_end(ap);
    }
}    /* cpLog */

#endif

void cpLog_impl_(int pri, const char* file, int line, const char* fmt, ...)
{
    if (pri < 0)
        return ;

    va_list ap;

    if (pri <= CpLogPriority::getPriority())
    {
        va_start(ap, fmt);
        vCpLog(pri, file, line, fmt, ap);
        va_end(ap);
    }
}

void
vCpLog(int pri, const char* file, int line, const char* fmt, va_list ap)
{
    if (pri < 0)
        return ;

    time_t now = 0;

    (void)time(&now);

    /* Use printf() for now */

    char datebuf[256];

    sprintf(datebuf, "%s", VTime::strftime("%y%m%d-%H%M%S").c_str());

    struct timeval tv =
        {
            0, 0
        };
    struct timezone tz =
        {
            0, 0
        };


    gettimeofday(&tv, &tz);

    char msbuf[50];

    sprintf(msbuf, ".%3.3ld", (tv.tv_usec / 1000));

    strcat(datebuf, msbuf);

    /* check for whether or not file or line is null! */
    char linebuf[256];
    if ((file == 0) || (line == 0))
    {
        strcpy(linebuf, "");
    }
    else
    {
        char filebuf[256];
        strcpy(filebuf, file);
        int x = strlen(filebuf) - 1;
        bool done = false;
        while (!done && (x >= 0))
        {
            if (filebuf[x] == '.')
            {
                filebuf[x] = '\0';
                done = true;
            }
            x--;
        }
        sprintf (linebuf, "%s:%d:", filebuf, line);
    }

    fprintf (cpLogFd, "%s [%5.5ld] %s %s: %.30s ",
             datebuf,
             VThread::selfId(),
             priNameShort[pri],
             CpLogPriority::getLabel(),
             linebuf
            );
    vfprintf (cpLogFd, fmt, ap);
    fprintf (cpLogFd, "\n");
    fflush(cpLogFd);
}    /* vCpLog */


void
cpLogSetPriority (int pri)
{
    if (pri < 0)
        return ;
    CpLogPriority::setPriority(pri);
}    /* cpLogSetPriority */

int
cpLogGetPriority ()
{
    return CpLogPriority::getPriority();
}    /* cpLogSetPriority */


void
cpLogSetPriorityThread (vthread_t thread_id, int pri)
{
    if (pri < 0)
        return ;
    CpLogPriority::setPriorityThread(thread_id, pri);
}


void
cpLogClearPriorityThread (vthread_t thread_id)
{
    CpLogPriority::clearPriorityThread(thread_id);
}


void
cpLogSetLabel (const char* label)
{
    CpLogPriority::setLabel(label);
    //    strncpy (cpLogLabel, label, LOG_LABEL_MAX_SIZE);
    //    cpLogLabel[LOG_LABEL_MAX_SIZE] = 0;
}    /* cpLogSetLabel */


void
cpLogSetLabelThread (vthread_t thread_id, const char* label)
{
    CpLogPriority::setLabelThread(thread_id, label);
}


void
cpLogClearLabelThread (vthread_t thread_id)
{
    CpLogPriority::clearLabelThread(thread_id);
}


void
cpLogShow (void)
{
    fprintf (stderr, "\tLabel    : %s\n", CpLogPriority::getLabel());
    fprintf (stderr, "\tPriority : %s\n", priName[CpLogPriority::getPriority()]);
    fprintf (stderr, "\tFile     : %s (cpLogFd = %d)\n", cpLogFile, fileno(cpLogFd));
}    /* cpLogShow */


int
cpLogStrToPriority(const char* priority)
{
    string pri = priority;

    if (pri.find("LOG_", 0) == 0)
    {
        pri.erase(0, 4);
    }

    int i = 0;
    while (priName[i] != 0)
    {
        if (pri == priName[i])
        {
            return i;
        }
        i++;
    }
    return -1;  // invalid
}


const char*
cpLogPriorityToStr(int priority)
{
    int priorityCount = 0;
    while (priName[priorityCount] != 0)
    {
        priorityCount++;
    }

    if ((priority >= 0) && (priority < priorityCount))
    {
        return priName[priority];
    }
    else
    {
        return 0;
    }
}


/* this function is not called, it is just compiled to make sure that
   the cpLog Macro expansion works */
void
testCpLogMacroExpansion()
{
    if (true)
        cpLog(LOG_DEBUG, "this is a test");
    else
        cpLog(LOG_DEBUG, "and a second");
}
