
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */




#ifndef CPLOG_H
#define CPLOG_H

static const char* const cpLogHeaderVersion =
    "$Id: cpLog.h,v 1.23 2000/10/04 22:37:58 veer Exp $";




#define __const const

#define USE_BETTER_CPLOG 1

#include <stdarg.h>
#include "VThread.h"

#ifndef LOG_EMERG
#define LOG_EMERG       0       /* system is unusable */
#define LOG_ALERT       1       /* action must be taken immediately */
#define LOG_CRIT        2       /* critical conditions */
#define LOG_ERR         3       /* error conditions */
#define LOG_WARNING     4       /* warning conditions */
#define LOG_NOTICE      5       /* normal but significant condition */
#define LOG_INFO        6       /* informational */
#define LOG_DEBUG       7       /* debug-level messages */
#define LOG_DEBUG_STACK 8       /* stack debug-level messages */
#define LOG_DEBUG_OPER  9       /* stack operator debug-level messages */
#define LOG_DEBUG_HB   10       /* heartbeat debug-level messages */
#endif

/*
 * tbd:
 *      cpLogClose(),
 *      remote log server
 *      multiple logs
 *      cpLogSetMask ()
 *      change attributes on the fly -> separate tasks
 *      open a new file when the current one is reaching a limit
 *      what else?
 *      ...
 */

#ifdef __cplusplus
extern "C" {
#endif

/* Generate a log message using FMT string and option arguments.  */

extern void cpLog_impl_ (int pri, const char* file, int line, const char* fmt, ...);
#ifdef __GNUC__
#if USE_BETTER_CPLOG
#define cpLog(priority__, fmt__, args__...) \
do {if (priority__ <= cpLogGetPriority()) \
cpLog_impl_ (priority__, __FILE__, __LINE__, fmt__ , ##args__);} \
while (0)
#else
#define cpLog(priority__, fmt__, args__...) \
cpLog_impl_ (priority__, __FILE__, __LINE__, fmt__ , ##args__)
#endif
#else
/* this is the regular definition */
extern void cpLog (int pri, const char *fmt, ...);
#endif

/* Generate a log message using FMT and using arguments pointed to by AP.  */
extern void vCpLog (int pri, const char* file, int line, const char *fmt, va_list ap);

extern void cpLogSetPriority (int pri);

extern int cpLogGetPriority ();

extern void cpLogSetPriorityThread (vthread_t thread_id, int pri);

extern void cpLogClearPriorityThread (vthread_t thread_id);

extern void cpLogSetLabelThread (vthread_t thread_id, const char* label);

extern void cpLogClearLabelThread (vthread_t thread_id);

extern void cpLogSetLabel (const char* label);

extern void cpLogShow (void);

extern bool cpLogOpen (const char* f);

extern int cpLogStrToPriority(const char* priority);

extern const char* cpLogPriorityToStr(int priority);

/* CPLOG_H */

#ifdef __cplusplus
}
#endif

#endif
