
/* ====================================================================
 * The Vovida Software License, Version 1.0 
 * 
 * Copyright (c) 2000 Vovida Networks, Inc.  All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. The names "VOCAL", "Vovida Open Communication Application Library",
 *    and "Vovida Open Communication Application Library (VOCAL)" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact vocal@vovida.org.
 *
 * 4. Products derived from this software may not be called "VOCAL", nor
 *    may "VOCAL" appear in their name, without prior written
 *    permission of Vovida Networks, Inc.
 * 
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, TITLE AND
 * NON-INFRINGEMENT ARE DISCLAIMED.  IN NO EVENT SHALL VOVIDA
 * NETWORKS, INC. OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT DAMAGES
 * IN EXCESS OF ,000, NOR FOR ANY INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
 * USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
 * DAMAGE.
 * 
 * ====================================================================
 * 
 * This software consists of voluntary contributions made by Vovida
 * Networks, Inc. and many individuals on behalf of Vovida Networks,
 * Inc.  For more information on Vovida Networks, Inc., please see
 * <http://www.vovida.org/>.
 *
 */
static const char* const DataTest_cxx_Version =
    "$Id: DataTest.cxx,v 1.8 2000/09/20 08:01:29 cvs Exp $";
#include "VTest.hxx"
#include "Data.hxx"

bool test_00()
{
    Data msg1("From: test1@test.com\r\nTo: test2@test2.com\r\nCall-ID: f81d4fae-7dec-11d0-a765-00a0c91e6bf6@foo.bar.com\r\n");
    Data msg2("From: test1@test.com\r\nTo: test2@test2.com\r\nCall-ID: e81d4fae-7dec-11d0-a765-00a0c91e6bf6@foo.bar.com\r\n");

    return !(msg1 < msg2);
}

bool test_01()
{
    bool pass = true;

    Data msg1("hello");
    Data msg2("Hello");

    if ( isEqualNoCase(msg1, msg2) != true )
    {
        pass = false;
    }

    Data msg3("HeLLo");
    Data msg4("hEllO");

    if ( isEqualNoCase(msg3, msg4) != true )
    {
        pass = false;
    }

    Data msg5("HeLLo");
    Data msg6("hEllOo");

    if ( isEqualNoCase(msg5, msg6) != false )
    {
        pass = false;
    }

    Data msg7("HeLLoo");
    Data msg8("hEllO");

    if ( isEqualNoCase(msg7, msg8) != false )
    {
        pass = false;
    }


    return pass;
}


bool test_02()
{
    const int MAX_TESTS = 25;

    Data data[MAX_TESTS];
    bool testPassed[MAX_TESTS];

    for (int i = 0; i < MAX_TESTS; i++)
    {
        testPassed[i] = false;
    }


    data[0] = "Testing Data";
    if (strcmp(data[0].getData(), "Testing Data") == 0)
    {
        testPassed[0] = true;
    }

    data[1] = data[0];
    if (strcmp(data[1].getData(), "Testing Data") == 0)
    {
        testPassed[1] = true;
    }

    data[2] = "Testing Data";
    data[2] += "added data";

    if (strcmp(data[2].getData(), "Testing Dataadded data") == 0)
    {
        testPassed[2] = true;
    }
    else
    {
        cerr << "test2: " << data[2].getData() << endl;
    }

    data[3] = data[1] + data[2];

    if (data[3] == "Testing DataTesting Dataadded data")
    {
        testPassed[3] = true;
    }
    else
    {
        cerr << "test3: " << data[3].getData() << endl;
    }

    string str("This is a string");
    Data data4(str);

    if (data4 == "This is a string")
    {
        testPassed[4] = true;
    }

    mstring mstr("This is a mstring");
    Data data5(mstr);

    if (data5 == "This is a mstring")
    {
        testPassed[5] = true;
    }

    int value = 5;
    Data data6(value);

    if (data6 == "5")
    {
        testPassed[6] = true;
    }

    Data testData;
    testData = "alphb";

    data[7] = "alpha";

    if ((data[7] < testData) &&
            !(data[7] == testData) &&
            !(data[7] > testData) &&
            (data[7] != testData))
    {
        testPassed[7] = true;
    }
    else
    {
        cerr << "test 7: " << endl;
        cerr << "x < testData : " << (data[7] < testData) << endl;
        cerr << "x > testData : " << (data[7] > testData) << endl;
        cerr << "x == testData : " << (data[7] == testData) << endl;
        cerr << "x != testData : " << (data[7] != testData) << endl;
    }

    data[8] = "alphc";
    if ((data[8] > testData) &&
            !(data[8] == testData) &&
            !(data[8] < testData) &&
            (data[8] != testData))
    {
        testPassed[8] = true;
    }
    else
    {
        cerr << "test 8: " << endl;
        cerr << "x < testData : " << (data[8] < testData) << endl;
        cerr << "x > testData : " << (data[8] > testData) << endl;
        cerr << "x == testData : " << (data[8] == testData) << endl;
        cerr << "x != testData : " << (data[8] != testData) << endl;
    }

    data[9] = "alphb";
    if ((data[9] == testData) &&
            !(data[9] < testData) &&
            !(data[9] > testData) &&
            !(data[9] != testData))
    {
        testPassed[9] = true;
    }
    else
    {
        cerr << "test 9: " << endl;
        cerr << "x < testData : " << (data[9] < testData) << endl;
        cerr << "x > testData : " << (data[9] > testData) << endl;
        cerr << "x == testData : " << (data[9] == testData) << endl;
        cerr << "x != testData : " << (data[9] != testData) << endl;
    }

    data[10] = "alphbx";

    if ((data[10] > testData) &&
            !(data[10] < testData) &&
            !(data[10] == testData) &&
            (data[10] != testData))
    {
        testPassed[10] = true;
    }
    else
    {
        cerr << "test 10: " << endl;
        cerr << "x < testData : " << (data[10] < testData) << endl;
        cerr << "x > testData : " << (data[10] > testData) << endl;
        cerr << "x == testData : " << (data[10] == testData) << endl;
        cerr << "x != testData : " << (data[10] != testData) << endl;
    }

    data[11] = "hello, world!";

    char newstr[256];
    strcpy(newstr, data[11].getData());

    if ((strcmp(newstr, "hello, world!") == 0) && (data[11].length() == 13))
    {
        testPassed[11] = true;
    }

    int myMax = 11;

    bool allPassed = true;

    cout << "START test program: " << __FILE__ << endl;

    for (int j = 0; j <= myMax; j++)
    {
        if (testPassed[j])
        {
            cout << "test " << j << " passed." << endl;
        }
        else
        {
            cout << "test " << j << " FAILED." << endl;
            allPassed = false;
        }
    }

    cout << "END test program: " << __FILE__ << endl;

    return allPassed;
}


bool test_03()
{
    Data one = "data one";
    Data two = "data two";

    Data three = one + two;
    bool ok = (three == "data onedata two");

    if (!ok)
    {
        cout << "three: " << three.getData() << endl;
    }
    return ok;
}


bool test_04()
{
    Data one = "data one";
    Data two = "data two";

    one += two;
    bool ok = (one == "data onedata two");

    if (!ok)
    {
        cout << "one: " << one.getData() << endl;
    }
    return ok;
}


bool test_05()
{
    Data one = "abcd";
    Data two = one;

    two.setchar(0, 'z');
    two.setchar(1, 'y');
    two.setchar(2, 'x');
    two.setchar(3, 'w');
    bool ok = (one == "abcd") && (two == "zyxw");
    if (!ok)
    {
        cout << "one: " << one.getData() << endl;
        cout << "two: " << two.getData() << endl;
    }
    return ok;
}


bool test_06()
{
    Data junk;
    return true;
}

bool test_07()
{
    Data junk = 7;
    if (junk == "7")
        return true;
    else
        return false;
}

bool test_08()
{
    Data junk;
    junk = 7;
    if (junk == "7")
        return true;
    else
        return false;
}

bool test_09()
{
    Data junk;
    junk = "Content-Length: ";
    junk += 7;
    if (junk == "Content-Length: 7")
        return true;
    else
        return false;
}

bool test_10()
{
    Data junk = "asnf abcd";
    Data rest;

    return (junk.match("d", &rest, true) == FOUND);
}

bool test_11()
{
    Data junk = "asnf abcd";
    Data rest;
    junk.match("d", &rest, true);
    cout << rest.getData() << endl;
    return (rest == "asnf abc");
}

bool test_12()
{
    Data test0 = "1024";
    Data test1 = "1000x1000";
    Data test2 = " 1000";
    Data test3 = "1000 1000";
    Data test4 = "19238471832758715289725371";
    Data test5 = "4000@";

    bool ok = true;

    if (!(int(test0) == 1024))
    {
        cout << "test12.0 failed" << endl;
        ok = false;
    }
    if (!(int(test1) == 1000))
    {
        cout << "test12.1 failed" << endl;
        ok = false;
    }
    if (!(int(test2) == 0))
    {
        cout << "test12.2 failed" << endl;
        ok = false;
    }
    if (!(int(test3) == 1000))
    {
        cout << "test12.3 failed" << endl;
        ok = false;
    }
    if (!(int(test5) == 4000))
    {
        cout << "test12.5 failed" << endl;
        ok = false;
    }

    int i = test4;
    cout << i << endl;

    return ok;
}

bool test_13()
{
    Data j = "asnf abcd";
    string s = j;
    return (Data(s) == j);
}


int main()
{
    VTest myTest(__FILE__);

    myTest.test(0, test_00() , "test for < operator");
    myTest.test(1, test_01() , "test for isEqualNoCase");
    myTest.test(2, test_02() , "general Data test");
    myTest.test(3, test_03() , "general Data test");
    myTest.test(4, test_04() , "general Data test");
    myTest.test(5, test_05() , "real copy on write behavior test");
    myTest.test(6, test_06() , "null object creation");
    myTest.test(7, test_07() , "number object creation");
    myTest.test(8, test_08() , "number object creation");
    myTest.test(9, test_09() , "number object creation");
    myTest.test(10, test_10() , "match test");
    myTest.test(11, test_11() , "match test");
    myTest.test(12, test_12() , "operator int test");
    myTest.test(13, test_13() , "operator string test");

    return myTest.exitCode();
}
